/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

/**
 * This is a private header of the channel connection API, exposing implementation details.
 * This header should not be exposed to applications.
 */

#ifndef PRIV_DOCA_CHANNEL_CONNECTION_H_
#define PRIV_DOCA_CHANNEL_CONNECTION_H_

#include <stdint.h>

#include <doca_error.h>
#include <priv_doca_compat.h>

#ifdef __cplusplus
extern "C" {
#endif

struct doca_dev;
struct priv_doca_channel_connection;

enum priv_doca_channel_connection_connection_type {
	PRIV_DOCA_CHANNEL_CONNECTION_CONNECT_BY_SERVICE_NAME = 0,
	PRIV_DOCA_CHANNEL_CONNECTION_CONNECT_BY_VHCA_ID = 1,
};

enum priv_doca_channel_connection_transport_type {
	PRIV_DOCA_CHANNEL_CONNECTION_UD = 0,
	PRIV_DOCA_CHANNEL_CONNECTION_DC = 1,
};

struct priv_doca_channel_connection_attributes {
	enum priv_doca_channel_connection_connection_type connection_type;
	enum priv_doca_channel_connection_transport_type transport_type;
	uint32_t qpn_or_dci;
	uint16_t dst_vhca_id;
	uint64_t token;
	const char *service_name;
};

/**
 * @brief Creates a CHANNEL_CONNECTION object
 *
 * @param [in] dev
 * doca device
 *
 * @param [in] channel connection attributes
 * priv_doca_channel_connection_attributes
 *
 * @param [out] channel_connection
 * channel connection object
 *
 * @return DOCA_SUCCESS, DOCA_ERROR_INVALID_VALUE, DOCA_ERROR_NO_MEMORY,
 * DOCA_ERROR_NOT_CONNECTED or DOCA_ERROR_DRIVER
 */
DOCA_PRIVATE
doca_error_t priv_doca_dev_channel_connection_create(const struct doca_dev *dev, const struct priv_doca_channel_connection_attributes *attr,
						     struct priv_doca_channel_connection **channel_connection);

/**
 * @brief Destroy CHANNEL_CONNECTION object
 *
 * @param [in] channel_connection
 *
 * @return doca_error_t
 */
DOCA_PRIVATE
doca_error_t priv_doca_dev_channel_connection_destroy(struct priv_doca_channel_connection *channel_connection);

/**
 * @brief this method returns the channel connection id
 * @param [in] channel_connection
 * @param [out] id
 * @return DOCA_SUCCESS, DOCA_ERROR_INVALID_VALUE or DOCA_ERROR_UNEXPECTED
 */
DOCA_PRIVATE
doca_error_t priv_doca_channel_connection_id_get(const struct priv_doca_channel_connection *channel_connection, uint32_t *id);

#ifdef __cplusplus
}
#endif

#endif /* PRIV_DOCA_CHANNEL_CONNECTION_H_ */
