/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

/**
 * This is a private header of the channel service API, exposing implementation details.
 * This header should not be exposed to applications.
 */

#ifndef PRIV_DOCA_CHANNEL_SERVICE_H_
#define PRIV_DOCA_CHANNEL_SERVICE_H_

#include <stdbool.h>
#include <stdint.h>

#include <doca_error.h>
#include <priv_doca_compat.h>

#ifdef __cplusplus
extern "C" {
#endif

struct doca_dev;
struct priv_doca_channel_service;

enum priv_doca_channel_service_transport_type {
	PRIV_DOCA_CHANNEL_SERVICE_UD = 0,
	PRIV_DOCA_CHANNEL_SERVICE_DC = 1,
};

struct priv_doca_channel_service_attributes {
	enum priv_doca_channel_service_transport_type transport_type;
	uint16_t allowed_vhca_id;
	bool enforce_token;
	uint32_t qpn_or_dct;
	uint32_t service_qkey;
	uint64_t token;
	uint64_t dct_key;
	const char *service_name;
};

/**
 * @brief Creates a CHANNEL_SERVICE object
 *
 * @param [in] dev
 * doca device
 *
 * @param [in] channel service attributes
 * priv_doca_channel_service_attributes
 *
 * @param [out] channel_service
 * channel service object
 *
 * @return DOCA_SUCCESS, DOCA_ERROR_INVALID_VALUE, DOCA_ERROR_NO_MEMORY,
 * DOCA_ERROR_NOT_CONNECTED or DOCA_ERROR_DRIVER
 */
DOCA_PRIVATE
doca_error_t priv_doca_dev_channel_service_create(const struct doca_dev *dev, const struct priv_doca_channel_service_attributes *attr,
						  struct priv_doca_channel_service **channel_service);

/**
 * @brief Destroy CHANNEL_SERVICE object
 *
 * @param [in] channel_service
 *
 * @return doca_error_t
 */
DOCA_PRIVATE
doca_error_t priv_doca_dev_channel_service_destroy(struct priv_doca_channel_service *channel_service);

/**
 * @brief this method returns the channel service id
 * @param [in] channel_service
 * @param [out] id
 * @return DOCA_SUCCESS, DOCA_ERROR_INVALID_VALUE or DOCA_ERROR_UNEXPECTED
 */
DOCA_PRIVATE
doca_error_t priv_doca_channel_service_id_get(const struct priv_doca_channel_service *channel_service, uint32_t *id);

#ifdef __cplusplus
}
#endif

#endif /* PRIV_DOCA_CHANNEL_SERVICE_H_ */
