/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

/**
 * This is a private header of the dynamically connected target API, exposing implementation details.
 * This header should not be exposed to applications.
 */

#ifndef PRIV_DOCA_DCT_H_
#define PRIV_DOCA_DCT_H_

#include <stdint.h>

#include <doca_error.h>
#include <priv_doca_compat.h>

#ifdef __cplusplus
extern "C" {
#endif

struct doca_dev;
struct priv_doca_cq;
struct priv_doca_rmp;
struct priv_doca_dct;

struct priv_doca_dct_attributes {
	struct priv_doca_cq *cq;
	struct priv_doca_rmp *rmp;
	uint8_t port;
	uint8_t mtu;
	uint8_t my_addr_index;
	uint8_t min_rnr_nak;
	uint64_t dc_access_key;
};

/**
 * @brief Creates a DCT object
 *
 * @param [in] dev
 * doca device
 *
 * @param [in] dct attributes
 * priv_doca_dct_attributes
 *
 * @param [out] dct
 * dct object
 *
 * @return DOCA_SUCCESS, DOCA_ERROR_INVALID_VALUE, DOCA_ERROR_NO_MEMORY,
 * DOCA_ERROR_NOT_CONNECTED, DOCA_ERROR_IO_FAILED or DOCA_ERROR_DRIVER
 */
DOCA_PRIVATE
doca_error_t priv_doca_dev_dct_create(const struct doca_dev *dev, const struct priv_doca_dct_attributes *attr,
				      struct priv_doca_dct **dct);

/**
 * @brief Destroy DCT object
 *
 * @param [in] dct
 *
 * @return doca_error_t
 */
DOCA_PRIVATE
doca_error_t priv_doca_dev_dct_destroy(struct priv_doca_dct *dct);

/**
 * @brief This method returns the dct id
 * @param [in] dct
 * @param [out] id
 * @return DOCA_SUCCESS, DOCA_ERROR_INVALID_VALUE or DOCA_ERROR_UNEXPECTED
 */
DOCA_PRIVATE
doca_error_t priv_doca_dct_id_get(const struct priv_doca_dct *dct, uint32_t *id);

#ifdef __cplusplus
}
#endif

#endif /* PRIV_DOCA_DCT_H_ */
