/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

/**
 * This is a private header of the hotplug API, exposing implementation details.
 * This header should not be exposed to applications.
 */

#ifndef PRIV_DOCA_HOTPLUG_H_
#define PRIV_DOCA_HOTPLUG_H_

#include <doca_error.h>
#include <priv_doca_compat.h>

#ifdef __cplusplus
extern "C" {
#endif

struct doca_dev_hotplug_attr;

/**
 * @brief Method for handling hotplug operation.
 *
 * @param [in] attr The attributes template that was constructed by user for hotplug operation.
 * @param [in] hotplug_dev_obj_len size in bytes of hotplug_dev_obj buffer
 * @param [out] hotplug_dev_obj A pointer to a pre-alloctaed buffer of PRM hotplug device object format
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * Error code - in case of failure. see doca_error_t.
 */
typedef doca_error_t (*priv_doca_dev_hotplug_t)(const struct doca_dev_hotplug_attr *attr, size_t hotplug_dev_obj_len, void *hotplug_dev_obj);

/**
 * @brief Hotplug attributes associated to DOCA dev.
 *
 */
struct doca_dev_hotplug_attr {
	struct doca_dev *dev;                             /**< Associated DOCA dev.*/
	priv_doca_dev_hotplug_t hotplug_dev_obj_build_cb; /**< Callback handling hotplug operation (filled by specific library).*/
};

#ifdef __cplusplus
}
#endif

#endif /* PRIV_DOCA_HOTPLUG_H_ */
