/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

/**
 * This is a private header of the user access region API, exposing implementation details.
 * This header should not be exposed to applications.
 */

#ifndef PRIV_DOCA_UAR_H_
#define PRIV_DOCA_UAR_H_

#include <stdint.h>

#include <doca_error.h>
#include <priv_doca_compat.h>

#ifdef __cplusplus
extern "C" {
#endif

struct doca_dev;
struct priv_doca_uar;

enum priv_doca_uar_allocation_type {
	PRIV_DOCA_UAR_ALLOCATION_TYPE_BLUEFLAME = 0,
	PRIV_DOCA_UAR_ALLOCATION_TYPE_NONCACHE = 1,
};

/**
 * @brief Creates a UAR object
 *
 * @param [in] dev
 * doca device
 *
 * @param [in] allocation type
 * priv_doca_uar_allocation_type
 *
 * @param [out] uar
 * uar object
 *
 * @return DOCA_SUCCESS, DOCA_ERROR_INVALID_VALUE or DOCA_ERROR_NO_MEMORY
 */
DOCA_PRIVATE
doca_error_t priv_doca_dev_uar_create(const struct doca_dev *dev, enum priv_doca_uar_allocation_type allocation_type,
				      struct priv_doca_uar **uar);

/**
 * @brief Destroy UAR object
 *
 * @param [in] uar
 */
DOCA_PRIVATE
void priv_doca_dev_uar_destroy(struct priv_doca_uar *uar);

/**
 * @brief this method returns the uar page id
 * @param [in] uar
 * @param [out] id
 * @return DOCA_SUCCESS, DOCA_ERROR_INVALID_VALUE or DOCA_ERROR_BAD_STATE
 */
DOCA_PRIVATE
doca_error_t priv_doca_uar_id_get(const struct priv_doca_uar *uar, uint32_t *id);

/**
 * @brief this method returns the uar page address
 * @param [in] uar
 * @param [out] page
 * @return DOCA_SUCCESS, DOCA_ERROR_INVALID_VALUE or DOCA_ERROR_BAD_STATE
 */
DOCA_PRIVATE
doca_error_t priv_doca_uar_page_get(const struct priv_doca_uar *uar, void **page);

#ifdef __cplusplus
}
#endif

#endif /* PRIV_DOCA_UAR_H_ */
