/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#pragma once

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <functional>
#include <unordered_map>
#include <vector>

#include <priv_doca_compat.h>
#include <doca_buf.h>

namespace doca_core
{

/**
 * This enum is used by the doca buf to get the offset from the map
 */
enum class buf_extension_type
{
	LINKED_LIST,
};

struct buf_extension_attributes {
	const buf_extension_type type;
	const doca_buf_extension extension_flag;
	const size_t size;
	const char *name;
};

using supported_extensions_attributes = std::vector<buf_extension_attributes>;

/**
 * @brief This method supplies the supported extensions
 * @return const reference to the supported extensions
 */
const supported_extensions_attributes &get_supported_extensions() noexcept;

/**
 * @brief This method checks if a set of extensions flags is valid
 * @param extension_flags [in] a set of extension flags
 * @return true if extensions are valid, false if not (flags containing unsupported flags)
 */
bool is_buf_extensions_valid(uint32_t extension_flags) noexcept;

/**
 * @brief This method calculates the required size for a set of extensions
 * @param extension_flags [in] a set of extension flags (can be 0)
 * @param attributes [in] extension attributes (default = operational map)
 * @return size required for the extensions in bytes
 */
size_t calculate_extensions_buffer_size(uint32_t extension_flags,
	const supported_extensions_attributes &attributes = get_supported_extensions()) noexcept;

} /* namespace doca_core */


/** @} */
