/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include <doca_error.h>
#include <doca_log.h>
#include "internal_doca_buf.h"
#include "buf_pool.hpp"

DOCA_LOG_REGISTER(CORE::BUF_POOL);

doca_core::buf_pool::buf_pool(uint32_t num_buffers, size_t extensions_buffer_size)
{
	auto doca_buf_size = (sizeof(doca_buf) + extensions_buffer_size);
	auto mem_area_size = (num_buffers * doca_buf_size);

	m_elements_buffer.resize(static_cast<size_t>(mem_area_size));
	m_elements_ptrs.resize(num_buffers);

	// Doca buffers end with variable size extension buffer so they have to be allocated using placement new.
	uint8_t *current = m_elements_buffer.data();
	for (size_t i = 0; i < num_buffers; i++) {
		m_elements_ptrs[i] = new (current) doca_buf();
		current += doca_buf_size;
	}
}

doca_core::buf_pool::~buf_pool()
{
	// buffers are allocated on m_elements_buffer using placement new, therefore their destructor must be explicitly
	// called.
	for (auto buf : m_elements_ptrs) {
		if (buf) {
			buf->~doca_buf();
		}
	}
}
