/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#pragma once

#include <memory>

#include <doca_error.h>
#include <imemory_region.hpp>
#include <iprotection_domain.hpp>
#include "memory_region_base.hpp"


namespace device_abstraction {

class alias_memory_region : public memory_region_base
{
public:

	/**
	 * @brief - This method allocates and initializes a alias_memory_region
	 * @param [in] devx adapter
	 * @param [in] protection domain
	 * @param [in] lkey
	 * @param [in] othe vhca id
	 * @param [in] access key
	 * @return create_mr_ret (see interface)
	 */
	static create_mr_ret create(std::shared_ptr<idevx_adapter> devx_adapter,
				    std::shared_ptr<iprotection_domain> protection_domain, uint32_t lkey,
				    uint16_t other_vhca_id, const allow_other_vhca_access_key_t &access_key) noexcept;

	/**
	 * Constructor
	 * @param [in] devx adapter
	 * @param [in] protection domain
	 */
	alias_memory_region(std::shared_ptr<idevx_adapter> devx_adapter,
			    std::shared_ptr<iprotection_domain> protection_domain) noexcept;

	/**
	 * Destructor
	 */
	virtual ~alias_memory_region() = default;

	/**
	 * brief - this method initializes the mkey
	 * @param [in] lkey
	 * @param [in] other vhca id
	 * @param [in] access key
	 * @return doca_error
	 */
	doca_error_t initialize(uint32_t lkey, uint16_t other_vhca_id,
				const allow_other_vhca_access_key_t &access_key) noexcept;

private:

	alias_memory_region(alias_memory_region const &) = delete;
	alias_memory_region &operator=(alias_memory_region const &) = delete;
	alias_memory_region(alias_memory_region &&) = delete;
	alias_memory_region &operator=(alias_memory_region &&) = delete;
};

} /* namespace device_abstraction */

