/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#pragma once

#include <idevice_adapter.hpp>
#include <doca_error.h>

namespace device_abstraction {

class device_adapter : public idevice_adapter
{
public:

	/**
	 * Destructor
	 */
	virtual ~device_adapter() = default;

	virtual query_hca_caps_ret query_hca_caps() noexcept override;

	virtual doca_error_t query_function_hca_caps(uint16_t function_id, hca_capabilities &caps) noexcept override;

	virtual doca_error_t query_hca_caps_by_opmod(uint16_t op_mod, void *out, size_t out_size) noexcept override;

	virtual create_completion_queue_ret create_completion_queue(std::shared_ptr<iuser_access_region> uar,
								    const icompletion_queue::attributes &attr,
								    union doca_data user_data,
								    priv_doca_cqe_event_handler_cb_t handler) noexcept override;

	virtual ichannel_service::create_ret
	create_channel_service(ichannel_service::attributes const &attr) noexcept override;

	virtual ichannel_connection::create_ret
	create_channel_connection(ichannel_connection::attributes const &attr) noexcept override;

	device_adapter(device_adapter const &) = delete;
	device_adapter &operator=(device_adapter const &) = delete;
	device_adapter(device_adapter &&) = delete;
	device_adapter &operator=(device_adapter &&) = delete;
protected:
	/**
	 * Constructor
	 */
	device_adapter() = default;
};

} /* namespace device_abstraction */


