/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#pragma once

#include <vector>
#include <memory>

#include <doca_error.h>

namespace device_abstraction {

class idevice_adapter;

/**
 * get_devices_ret
 * @ error:
 *   - DOCA_SUCCESS if got devices list
 *   - DOCA_ERROR_NOT_FOUND if failed to get devices list
 *   - DOCA_ERROR_NO_MEMORY if failed to allocate memory.
 * @ devices
 */
using get_devices_ret = std::pair<doca_error_t, std::vector<std::shared_ptr<idevice_adapter>>>;

/**
 * @brief This method creates a set of all available devices
 * @return get_devices_ret (see above)
 */
get_devices_ret get_devices() throw();

} /* namespace device_abstraction */

