/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#pragma once

#include <stdint.h>
#include <utility>
#include <memory>

namespace device_abstraction {

class iuser_access_region
{
public:
	enum class allocation_type {
		blueflame = 0,
		noncache = 1,
	};

	/**
	 * @brief This pair is used as the return value for creating a user access region
	 * @first: doca_error_t
	 *         DOCA_SUCCESS
	 *         DOCA_ERROR_NO_MEMORY
	 *         DOCA_ERROR_DRIVER (devx failure)
	 * @second: shared pointer of the user access region.
	 */
	using create_ret = std::pair<doca_error_t, std::shared_ptr<iuser_access_region>>;

	/**
	 * Destructor
	 * Note: concrete destructor will attempt to destroy the user access region.
	 */
	virtual ~iuser_access_region() = default;

	/**
	 * @brief this method returns the uar page id
	 * return uint32_t
	 */
	virtual uint32_t get_id() const noexcept = 0;

	/**
	 * @brief this method returns the uar page address
	 * return void *
	 */
	virtual void *get_page() const noexcept = 0;

	iuser_access_region(iuser_access_region const &) = delete;
	iuser_access_region &operator=(iuser_access_region const &) = delete;
	iuser_access_region(iuser_access_region &&) = delete;
	iuser_access_region &operator=(iuser_access_region &&) = delete;

protected:
	/**
	 * Constructor
	 */
	iuser_access_region() noexcept = default;
};

} /* namespace device_abstraction */
