/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#pragma once

#include <memory>
#include <vector>

#include <doca_error.h>
#include <doca_types.h>
#include <priv_doca_types.h>

#include <icompletion_events_channel.hpp>

struct ibv_context;
struct mlx5dv_devx_event_channel;

namespace device_abstraction {

class linux_completion_events_channel : public icompletion_events_channel
{
  public:

	static create_comp_events_channel_ret create(struct ibv_context *ctx, bool is_blocking) noexcept;

	/**
	 * Constructor
	 */
	linux_completion_events_channel() noexcept = default;

	/**
	 * Destructor
	 */
	virtual ~linux_completion_events_channel();

	virtual doca_error_t initialize(struct ibv_context *ctx, bool is_blocking) noexcept;

	virtual doca_error_t register_cq(std::shared_ptr<icompletion_queue> cq) noexcept override;

	virtual doca_error_t unregister_cq(std::shared_ptr<icompletion_queue> cq) noexcept override;

	virtual void arm() noexcept override;

	virtual void force_arm() noexcept override;

	virtual doca_error_t handle_events(doca_event_handle_t handle = 0) noexcept override;

	virtual doca_event_handle_t get_handle() const noexcept override;

	linux_completion_events_channel(linux_completion_events_channel const &) = delete;
	linux_completion_events_channel &operator=(linux_completion_events_channel const &) = delete;
	linux_completion_events_channel(linux_completion_events_channel &&) = delete;
	linux_completion_events_channel &operator=(linux_completion_events_channel &&) = delete;
  protected:

	struct cookie {
		icompletion_queue *fast_cq{nullptr};
		bool is_armed{false};
		std::shared_ptr<icompletion_queue> cq{nullptr};
	};

	std::vector<linux_completion_events_channel::cookie*>::iterator find_cq(std::shared_ptr<icompletion_queue> cq);

	struct mlx5dv_devx_event_channel *m_channel{nullptr};
	std::vector<linux_completion_events_channel::cookie*> m_registered_cqs;
	bool m_is_blocking{false};
};

} // namespace device_abstraction
