/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#pragma once

#include <vector>
#include <unordered_map>
#include <utility>

#include <stdint.h>
#include <sys/epoll.h>
#include <sys/eventfd.h>

#include <doca_types.h>
#include <priv_doca_types.h>

#include <ievents_mux.hpp>

namespace device_abstraction {

class linux_events_mux : public ievents_mux
{
  public:
	linux_events_mux() noexcept = default;

	virtual ~linux_events_mux();

	/**
	 * @brief Initializes the mux for usage, operation may fail.
	 * @return doca_error_t
	 */
	virtual doca_error_t initialize() noexcept;

	virtual doca_error_t destroy() noexcept override;

	virtual doca_error_t register_handle(doca_event_handle_t handle, const struct priv_doca_event_handle_ops& ops,
					     union doca_data ops_data) noexcept override;

	virtual doca_error_t unregister_handle(doca_event_handle_t handle) noexcept override;

	virtual doca_error_t arm() noexcept override;

	virtual doca_error_t clear(doca_event_handle_t) noexcept override;

	virtual doca_event_handle_t get_handle() const noexcept override;

	virtual doca_error_t trigger_event() noexcept override;

	virtual doca_error_t untrigger_event() noexcept override;

	linux_events_mux(linux_events_mux const &) = delete;
	linux_events_mux &operator=(linux_events_mux const &) = delete;
	linux_events_mux(linux_events_mux &&) = delete;
	linux_events_mux &operator=(linux_events_mux &&) = delete;

  protected:
	using event_handle_cookie = std::pair<struct priv_doca_event_handle_ops, union doca_data>;
	std::unordered_map<doca_event_handle_t, event_handle_cookie> registered_handles;
	std::vector<struct epoll_event> events_buffer;
	int epoll_fd{-1};
	int manual_fd{-1};
};

} // namespace device_abstraction
