/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#pragma once

#include <memory>

#include <infiniband/mlx5dv.h>

#include <doca_error.h>

#include <protection_domain.hpp>

namespace device_abstraction {

class idevx_object;
class linux_devx_adapter;

class linux_protection_domain : public protection_domain
{
public:

	/**
	 * brief - this method allocates and initializes a pd
	 * @param [in] devx adapter
	 * @return pd or null
	 */
	static std::shared_ptr<linux_protection_domain>
	create(std::shared_ptr<linux_devx_adapter> devx_adapter) noexcept;

	/**
	 * Constructor
	 * @param [in] devx adapter
	 */
	linux_protection_domain(std::shared_ptr<linux_devx_adapter> devx_adapter) noexcept;

	/**
	 * Destructor
	 */
	virtual ~linux_protection_domain();

	/**
	 * brief - this method initializes the pd
	 * @return doca_error_t
	 */
	doca_error_t initialize() noexcept;

	/**
	 * @brief This method explicitly destroys the protection domain
	 * @return doca_error_t
	 */
	virtual doca_error_t destroy() noexcept override;

	/**
	 * brief this method returns the pd id
	 * return uint32_t
	 */
	virtual uint32_t get_pd_id() const noexcept override;

	/**
	 * @brief This method returns the ibv pd.
	 * @return ibv_pd
	 */
	virtual struct ibv_pd *get_ibv_pd() const noexcept override;

	linux_protection_domain(linux_protection_domain const &) = delete;
	linux_protection_domain &operator=(linux_protection_domain const &) = delete;
	linux_protection_domain(linux_protection_domain &&) = delete;
	linux_protection_domain &operator=(linux_protection_domain &&) = delete;

private:

	struct ibv_pd *m_pd_obj{ nullptr };
	uint32_t m_pd{ 0 };
};

} /* namespace device_abstraction */

