/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#pragma once

#include <memory>

#include <infiniband/mlx5dv.h>

#include <doca_error.h>
#include <iuser_access_region.hpp>

namespace device_abstraction {

class linux_user_access_region : public iuser_access_region
{
public:

	/**
	 * @brief this method allocates and initializes a uar
	 * @param [in] ibv context
	 * @param [in] uar allocation type
	 * @return iuser_access_region::create_ret (see interface)
	 */
	static iuser_access_region::create_ret
	create(struct ibv_context *ibv_ctx, iuser_access_region::allocation_type allocation_type) noexcept;

	/**
	 * Constructor
	 */
	linux_user_access_region() noexcept = default;

	/**
	 * Destructor
	 */
	virtual ~linux_user_access_region();

	/**
	 * @brief this method initializes the uar
	 * @param [in] ibv context
	 * @param [in] uar allocation type
	 * @return doca_error_t
	 */
	doca_error_t
	initialize(struct ibv_context *ibv_ctx, iuser_access_region::allocation_type allocation_type) noexcept;

	/**
	 * @brief this method returns the uar page id
	 * return uint32_t
	 */
	virtual uint32_t get_id() const noexcept override;

	/**
	 * @brief this method returns the uar page address
	 * return void *
	 */
	virtual void *get_page() const noexcept override;

	linux_user_access_region(linux_user_access_region const &) = delete;
	linux_user_access_region &operator=(linux_user_access_region const &) = delete;
	linux_user_access_region(linux_user_access_region &&) = delete;
	linux_user_access_region &operator=(linux_user_access_region &&) = delete;

private:

	mlx5dv_devx_uar *m_uar_obj{ nullptr };
};

} /* namespace device_abstraction */
