/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#pragma once

#include <memory>

#include <doca_error.h>
#include <imapped_user_memory.hpp>

namespace device_abstraction {

class mapped_user_memory : public imapped_user_memory
{
public:
	/**
	 * Destructor
	 */
	virtual ~mapped_user_memory() = default;

	/**
	 * brief This method returns the mapped_user_memory size
	 * return size_t
	 */
	inline virtual size_t get_size () const noexcept override
	{
		return m_size;
	}

	/**
	 * brief This method returns the mapped_user_memory buffer
	 * return void *
	 */
	inline virtual void* get_buffer () const noexcept override
	{
		return m_address;
	}

	/**
	 * brief This method returns the access flags
	 * @return user_memory_access
	 */
	inline virtual uint32_t get_access_flags () const noexcept override
	{
		return m_access_flags;
	}

	/**
	 * @brief This method returns the page size set for this mapped memory
	 * @return size_t
	 */
	inline virtual size_t get_page_size () const noexcept override
	{
		return m_page_size;
	}

	mapped_user_memory(mapped_user_memory const &) = delete;
	mapped_user_memory &operator=(mapped_user_memory const &) = delete;
	mapped_user_memory(mapped_user_memory &&) = delete;
	mapped_user_memory &operator=(mapped_user_memory &&) = delete;

protected:

	/**
	 * Constructor
	 * @param [in] address
	 * @param [in] size
	 * @param [in] page size
	 * @param [in] access flags
	 */
	mapped_user_memory(void *address, size_t size, size_t page_size, uint32_t access_flags) noexcept;

	void *m_address{ nullptr };
	const size_t m_size;
	const size_t m_page_size;
	const uint32_t m_access_flags;
};

} /* namespace device_abstraction */
