/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include <doca_log.h>
#include <memory_region.hpp>
#include <idevx_adapter.hpp>
#include "memory_region_base.hpp"

using namespace device_abstraction;

DOCA_LOG_REGISTER(CORE::MEMROY_REGION_BASE);

memory_region_base::memory_region_base(std::shared_ptr<idevx_adapter> devx_adapter,
					std::shared_ptr<iprotection_domain> pd) noexcept
: m_devx_adapter(devx_adapter), m_pd(pd)
{
}

memory_region_base::~memory_region_base()
{
	destroy_key();
}

doca_error_t
memory_region_base::destroy_key() noexcept
{
	if (m_mkey) {
		auto status = m_mkey->destroy();
		if (status != DOCA_SUCCESS) {
			DOCA_DLOG_ERR("Failed to destroy mkey (id = %d)", m_key_index);
			return status;
		}

		m_mkey.reset();
		m_key_index = 0;

		// Once the mkey is destroyed there is no need for the protection domain reference.
		// (and the reference can be reset).
		m_pd.reset();
	}

	return DOCA_SUCCESS;
}

uint32_t
memory_region_base::get_id() const noexcept
{
	return m_key_index;
}

doca_error_t
memory_region_base::allow_other_vhca_access(const allow_other_vhca_access_key_t &access_key) noexcept
{
	return DOCA_ERROR_NOT_SUPPORTED;
}

