/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#pragma once

#include <memory>

#include <doca_error.h>

#include <imemory_region.hpp>
#include <iprotection_domain.hpp>

namespace device_abstraction {

class idevx_adapter;
class idevx_object;

class memory_region_base : public imemory_region
{
public:
	/**
	 * Destructor
	 */
	virtual ~memory_region_base();

	/**
	 * @brief This method explicitly destroys the key. Destruction may fail if the mkey is still used.
	 * @return doca_error_t
	 */
	virtual doca_error_t destroy_key() noexcept override;

	/**
	 * This method returns the mkey index
	 * @return uint32_t
	 */
	virtual uint32_t get_id() const noexcept override;

	/**
	 * @brief This method allows other VHCA access
	 * @param [in] access key
	 * @param [in] access key size
	 * @return doca_error_t
	 */
	virtual doca_error_t allow_other_vhca_access(const allow_other_vhca_access_key_t &access_key) noexcept override;

protected:

	/**
	 * Constructor
	 */
	memory_region_base(std::shared_ptr<idevx_adapter> devx_adapter,
			   std::shared_ptr<iprotection_domain> pd) noexcept;

	std::shared_ptr<idevx_adapter> m_devx_adapter{nullptr};
	std::shared_ptr<iprotection_domain> m_pd{nullptr};
	std::unique_ptr<idevx_object> m_mkey{nullptr};
	uint32_t m_key_index{0};

private:

	memory_region_base(memory_region_base const &) = delete;
	memory_region_base &operator=(memory_region_base const &) = delete;
	memory_region_base(memory_region_base &&) = delete;
	memory_region_base &operator=(memory_region_base &&) = delete;
};

} /* namespace device_abstraction */

