/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#pragma once

#include <memory>

#include <doca_error.h>
#include <iprotection_domain.hpp>
#include <ireceive_memory_pool.hpp>
#include <idynamically_connected_target.hpp>

namespace device_abstraction {

class idevx_adapter;
class idevx_object;

class protection_domain : public iprotection_domain, public std::enable_shared_from_this<protection_domain>
{
public:

	protection_domain(std::shared_ptr<idevx_adapter> devx_adapter) noexcept;

	virtual ~protection_domain() = default;

	virtual create_mr_ret
	create_mapped_memory_region(void *address, size_t size, size_t page_size,
				    uint32_t access_flags) noexcept override;

	virtual create_mr_ret
	create_zero_based_mapped_memory_region(void *address, size_t size, size_t page_size,
					       uint32_t access_flags) noexcept override;

	virtual create_mr_ret
	create_alias_memory_region(uint32_t lkey, uint16_t vhca_id,
				   const allow_other_vhca_access_key_t &access_key) noexcept override;

	virtual create_mr_ret
	create_crossing_memory_region(uint32_t introspection_mkey, uint16_t vhca_id) noexcept override;

	virtual ireceive_memory_pool::create_ret
	create_receive_memory_pool(uint32_t wq_size) noexcept override;

	virtual create_mr_ret
	create_indirect_memory_region(uint32_t num_regions) noexcept override;

	virtual create_umr_ret
	create_user_memory_registration(uint32_t num_operations) noexcept override;

	virtual idynamically_connected_target::create_ret
	create_dynamically_connected_target(idynamically_connected_target::attributes const &attr) noexcept override;

	protection_domain(protection_domain const &) = delete;
	protection_domain &operator=(protection_domain const &) = delete;
	protection_domain(protection_domain &&) = delete;
	protection_domain &operator=(protection_domain &&) = delete;

protected:
	std::shared_ptr<idevx_adapter> m_devx_adapter{ nullptr };
};

} /* namespace device_abstraction */

