/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef MLX5_DEVX_H_
#define MLX5_DEVX_H_

#include <stddef.h>
#include <stdint.h>
#include <endian.h>

#include "mlx5_prm.h"

#ifdef __cplusplus
extern "C" {
#endif

#define __mlx5_nullp(typ) ((struct mlx5_ifc_##typ##_bits *)NULL)
#define __mlx5_st_sz_bits(typ) sizeof(struct mlx5_ifc_##typ##_bits)
#define __mlx5_bit_sz(typ, fld) sizeof(__mlx5_nullp(typ)->fld)
#define __mlx5_bit_off(typ, fld) offsetof(struct mlx5_ifc_##typ##_bits, fld)
#define __mlx5_dw_off(bit_off) ((bit_off) / 32)
#define __mlx5_64_off(bit_off) ((bit_off) / 64)
#define __mlx5_dw_bit_off(bit_sz, bit_off) (32 - (bit_sz) - ((bit_off) & 0x1f))
#define __mlx5_mask(bit_sz) ((uint32_t)((1ull << (bit_sz)) - 1))
#define __mlx5_dw_mask(bit_sz, bit_off)                                        \
	(__mlx5_mask(bit_sz) << __mlx5_dw_bit_off(bit_sz, bit_off))

#define MLX5_FLD_SZ_BYTES(typ, fld) (__mlx5_bit_sz(typ, fld) / 8)
#define MLX5_ST_SZ_BYTES(typ) (sizeof(struct mlx5_ifc_##typ##_bits) / 8)
#define MLX5_ST_SZ_DW(typ) (sizeof(struct mlx5_ifc_##typ##_bits) / 32)
#define MLX5_ST_SZ_QW(typ) (sizeof(struct mlx5_ifc_##typ##_bits) / 64)
#define MLX5_UN_SZ_BYTES(typ) (sizeof(union mlx5_ifc_##typ##_bits) / 8)
#define MLX5_UN_SZ_DW(typ) (sizeof(union mlx5_ifc_##typ##_bits) / 32)
#define MLX5_BYTE_OFF(typ, fld) (__mlx5_bit_off(typ, fld) / 8)
#define MLX5_ADDR_OF(typ, p, fld)                                              \
	((unsigned char *)(p) + MLX5_BYTE_OFF(typ, fld))

static inline void _mlx5_set(void *p, uint32_t value, size_t bit_off,
			     size_t bit_sz)
{
	__be32 *fld = (__be32 *)(p) + __mlx5_dw_off(bit_off);
	uint32_t dw_mask = __mlx5_dw_mask(bit_sz, bit_off);
	uint32_t mask = __mlx5_mask(bit_sz);

	*fld = htobe32((be32toh(*fld) & (~dw_mask)) |
		       ((value & mask) << __mlx5_dw_bit_off(bit_sz, bit_off)));
}

#define MLX5_SET(typ, p, fld, v)                                               \
	_mlx5_set(p, v, __mlx5_bit_off(typ, fld), __mlx5_bit_sz(typ, fld))

static inline uint32_t _mlx5_get(const void *p, size_t bit_off, size_t bit_sz)
{
	return ((be32toh(*((const __be32 *)(p) + __mlx5_dw_off(bit_off))) >>
		 __mlx5_dw_bit_off(bit_sz, bit_off)) &
		__mlx5_mask(bit_sz));
}

#define MLX5_GET(typ, p, fld)                                                  \
	_mlx5_get(p, __mlx5_bit_off(typ, fld), __mlx5_bit_sz(typ, fld))

static inline void _mlx5_set64(void *p, uint64_t v, size_t bit_off)
{
	*((__be64 *)(p) + __mlx5_64_off(bit_off)) = htobe64(v);
}

#define MLX5_SET64(typ, p, fld, v) _mlx5_set64(p, v, __mlx5_bit_off(typ, fld))

static inline uint64_t _mlx5_get64(const void *p, size_t bit_off)
{
	return be64toh(*((const __be64 *)(p) + __mlx5_64_off(bit_off)));
}

#define MLX5_GET64(typ, p, fld) _mlx5_get64(p, __mlx5_bit_off(typ, fld))

#ifdef __cplusplus
}
#endif

#endif /* MLX5_DEVX_H_ */
