/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef MLX5_PRM_H_
#define MLX5_PRM_H_

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef u8
#define u8 uint8_t
#endif

/*******************************************************************************
 * Objects related to commands - PRM section 29 Command Reference
 ******************************************************************************/

enum {
	MLX5_CMD_OP_QUERY_HCA_CAP = 0x100,
	MLX5_CMD_OP_QUERY_ISSI = 0x10a,
	MLX5_CMD_OP_SET_ISSI = 0x10b,
	MLX5_CMD_OP_CREATE_MKEY = 0x200,
	MLX5_CMD_OP_CREATE_CQ = 0x400,
	MLX5_CMD_OP_CREATE_QP = 0x500,
	MLX5_CMD_OP_DESTROY_QP = 0x501,
	MLX5_CMD_OP_RST2INIT_QP = 0x502,
	MLX5_CMD_OP_INIT2RTR_QP = 0x503,
	MLX5_CMD_OP_RTR2RTS_QP = 0x504,
	MLX5_CMD_OP_RTS2RTS_QP = 0x505,
	MLX5_CMD_OP_SQERR2RTS_QP = 0x506,
	MLX5_CMD_OP_2ERR_QP = 0x507,
	MLX5_CMD_OP_2RST_QP = 0x50a,
	MLX5_CMD_OP_QUERY_QP = 0x50b,
	MLX5_CMD_OP_SQD2RTS_QP = 0x50c,
	MLX5_CMD_OP_INIT2INIT_QP = 0x50e,
	MLX5_CMD_OP_CREATE_DCT = 0x710,
	MLX5_CMD_OP_DESTROY_DCT = 0x711,
	MLX5_CMD_OP_DRAIN_DCT = 0x712,
	MLX5_CMD_OP_QUERY_DCT = 0x713,
	MLX5_CMD_OP_ARM_DCT_FOR_KEY_VIOLATION = 0x714,
	MLX5_CMD_OP_QUERY_ESW_FUNCTIONS = 0x740,
	MLX5_CMD_OP_ALLOC_PD = 0x800,
	MLX5_CMD_OP_CREATE_RMP = 0x90C,
	MLX5_CMD_OP_CREATE_GENERAL_OBJECT = 0xA00,
	MLX5_CMD_OP_MODIFY_GENERAL_OBJECT = 0xA01,
	MLX5_CMD_OP_QUERY_GENERAL_OBJECT = 0xA02,
	MLX5_CMD_OP_DESTROY_GENERAL_OBJECT = 0xA03,
	MLX5_CMD_OP_QUERY_EMULATED_FUNCTIONS_INFO = 0xB03,
	MLX5_CMD_OP_ALLOW_OTHER_VHCA_ACCESS = 0xB16,
	MLX5_CMD_OP_HOTPLUG_DEVICE = 0xB20,
	MLX5_CMD_OP_HOTUNPLUG_DEVICE = 0xB21,
	MLX5_CMD_OP_QUERY_VUID = 0xB22,
};

enum {
	MLX5_CMD_STAT_OK = 0x0,
	MLX5_CMD_STAT_INT_ERR = 0x1,
	MLX5_CMD_STAT_BAD_OP_ERR = 0x2,
	MLX5_CMD_STAT_BAD_PARAM_ERR = 0x3,
	MLX5_CMD_STAT_BAD_SYS_STATE_ERR = 0x4,
	MLX5_CMD_STAT_BAD_RES_ERR = 0x5,
	MLX5_CMD_STAT_RES_BUSY = 0x6,
	MLX5_CMD_STAT_LIM_ERR = 0x8,
	MLX5_CMD_STAT_BAD_RES_STATE_ERR = 0x9,
	MLX5_CMD_STAT_IX_ERR = 0xa,
	MLX5_CMD_STAT_NO_RES_ERR = 0xf,
	MLX5_CMD_STAT_BAD_INP_LEN_ERR = 0x50,
	MLX5_CMD_STAT_BAD_OUTP_LEN_ERR = 0x51,
	MLX5_CMD_STAT_BAD_QP_STATE_ERR = 0x10,
	MLX5_CMD_STAT_BAD_PKT_ERR = 0x30,
	MLX5_CMD_STAT_BAD_SIZE_OUTS_CQES_ERR = 0x40,
};

enum {
	MLX5_QPC_ST_RC = 0x0,
	MLX5_QPC_ST_UC = 0x1,
	MLX5_QPC_ST_UD = 0x2,
	MLX5_QPC_ST_XRC = 0x3,
	MLX5_QPC_ST_DCI = 0x5,
	MLX5_QPC_ST_QP0 = 0x7,
	MLX5_QPC_ST_QP1 = 0x8,
	MLX5_QPC_ST_RAW_DATAGRAM = 0x9,
	MLX5_QPC_ST_REG_UMR = 0xc,
};

/* The timestamp format attributes to configure queues (RQ/SQ/QP). */
enum mlx5_qpc_timestamp_format {
	MLX5_QPC_TIMESTAMP_FORMAT_FREE_RUNNING = 0x0,
	MLX5_QPC_TIMESTAMP_FORMAT_DEFAULT = 0x1,
	MLX5_QPC_TIMESTAMP_FORMAT_REAL_TIME = 0x2,
};

enum {
	MLX5_QPC_MTU_256_BYTES = 0x1,
	MLX5_QPC_MTU_512_BYTES = 0x2,
	MLX5_QPC_MTU_1K_BYTES = 0x3,
	MLX5_QPC_MTU_2K_BYTES = 0x4,
	MLX5_QPC_MTU_4K_BYTES = 0x5,
	MLX5_QPC_MTU_RAW_ETHERNET_QP = 0x7,
};

enum {
	MLX5_QPC_PM_STATE_MIGRATED = 0x3,
};

enum {
	MLX5_GENERAL_OBJ_TYPE_CHANNEL_SERVICE = 0x3B,
	MLX5_GENERAL_OBJ_TYPE_CHANNEL_CONNECTION = 0x3C,
};

#define MLX5_GENERAL_OBJ_TYPES_CAP_CHANNEL_SERVICE     \
    (1ULL << MLX5_GENERAL_OBJ_TYPE_CHANNEL_SERVICE)

#define MLX5_GENERAL_OBJ_TYPES_CAP_CHANNEL_CONNECTION  \
    (1ULL << MLX5_GENERAL_OBJ_TYPE_CHANNEL_CONNECTION)

enum {
	MLX5_CAP_PORT_TYPE_IB = 0x0,
	MLX5_CAP_PORT_TYPE_ETH = 0x1,
};

/* PRM section 29.3.4.1 HCA Device Capabilities */

struct mlx5_ifc_cmd_hca_cap_bits {
	u8         access_other_hca_roce[0x1];
	u8         alloc_flow_counter_pd[0x1];
	u8         flow_counter_access_aso[0x1];
	u8         reserved_at_3[0x5];
	u8         flow_counter_acess_aso_opc_mod[0x8];
	u8         shared_object_to_user_object_allowed[0x1];
	u8         reserved_at_11[0x2];
	u8         log_sha_mmo_max_size[0x5];
	u8         reserved_at_18[0x7];
	u8         vhca_resource_manager[0x1];

	u8         hca_cap_2[0x1];
	u8         create_lag_when_not_master_up[0x1];
	u8         dtor[0x1];
	u8         event_on_vhca_state_teardown_request[0x1];
	u8         event_on_vhca_state_in_use[0x1];
	u8         event_on_vhca_state_active[0x1];
	u8         event_on_vhca_state_allocated[0x1];
	u8         event_on_vhca_state_invalid[0x1];
	u8         transpose_max_element_size[0x8];
	u8         vhca_id[0x10];

	u8         transpose_max_cols[0x8];
	u8         transpose_max_rows[0x8];
	u8         transpose_max_size[0x10];

	u8         resources_on_virtio_fs_emulation_manager[0x1];
	u8         sw_steering_icm_large_scale_steering[0x1];
	u8         qp_data_in_order[0x1];
	u8         log_regexp_scatter_gather_size[0x5];
	u8         ibl2_delay_drop[0x1];
	u8         ibl2_qp[0x1];
	u8         init2init_qp_drain_sigerr[0x1];
	u8         log_dma_mmo_max_size[0x5];
	u8         relaxed_ordering_write_pci_enabled[0x1];
	u8         ibl2_tx_vl15[0x1];
	u8         decompress_snappy[0x1];
	u8         log_compress_max_size[0x5];
	u8         decompress_lz4_data_only[0x1];
	u8         decompress_lz4_no_checksum[0x1];
	u8         decompress_lz4_checksum[0x1];
	u8         log_decompress_max_size[0x5];

	u8         log_max_srq_sz[0x8];
	u8         log_max_qp_sz[0x8];
	u8         event_cap[0x1];
	u8         rc_qp_init2rts[0x1];
	u8         rc_qp_create_on_init[0x1];
	u8         isolate_vl_tc_new[0x1];
	u8         comm_channel_dc[0x1];
	u8         comm_channel_cross_esw[0x1];
	u8         nvmeotcp[0x1];
	u8         pcie_hanged[0x1];
	u8         prio_tag_required[0x1];
	u8         wqe_index_ignore_cap[0x1];
	u8         dynamic_qp_allocation[0x1];
	u8         log_max_qp[0x5];

	u8         must_be_0[0x1];
	u8         reserved_at_a1[0x1];
	u8         regexp_alloc_onbehalf_umem[0x1];
	u8         ece[0x1];
	u8         regexp_num_of_engines[0x4];
	u8         allow_pause_tx[0x1];
	u8         reg_c_preserve[0x1];
	u8         reserved_at_aa[0x1];
	u8         log_max_srq[0x5];
	u8         nisp[0x1];
	u8         uplink_follow[0x1];
	u8         ts_cqe_to_dest_cqn[0x1];
	u8         reserved_at_b3[0x5];
	u8         selective_repeat[0x1];
	u8         go_back_n[0x1];
	u8         shampo[0x1];
	u8         scatter_fcs_w_decap_disable[0x1];
	u8         reserved_at_bc[0x4];

	u8         max_sgl_for_optimized_performance[0x8];
	u8         log_max_cq_sz[0x8];
	u8         reserved_at_d0[0x2];
	u8         access_register_user[0x1];
	u8         max_num_of_fixed_buffers_per_mkey[0x5];
	u8         upt_device_emulation_manager[0x1];
	u8         virtio_net_device_emulation_manager[0x1];
	u8         virtio_blk_device_emulation_manager[0x1];
	u8         log_max_cq[0x5];

	u8         log_max_eq_sz[0x8];
	u8         relaxed_ordering_write[0x1];
	u8         relaxed_ordering_read[0x1];
	u8         log_max_mkey[0x6];
	u8         tunneled_atomic[0x1];
	u8         as_notify[0x1];
	u8         m_pci_port[0x1];
	u8         m_vhca_mk[0x1];
	u8         hotplug_manager[0x1];
	u8         nvme_device_emulation_manager[0x1];
	u8         terminate_scatter_list_mkey[0x1];
	u8         repeated_mkey[0x1];
	u8         dump_fill_mkey[0x1];
	u8         dpp[0x1];
	u8         resources_on_nvme_emulation_manager[0x1];
	u8         fast_teardown[0x1];
	u8         log_max_eq[0x4];

	u8         max_indirection[0x8];
	u8         fixed_buffer_size[0x1];
	u8         log_max_mrw_sz[0x7];
	u8         force_teardown[0x1];
	u8         hard_coded_1[0x1];
	u8         log_max_bsf_list_size[0x6];
	u8         umr_extended_translation_offset[0x1];
	u8         null_mkey[0x1];
	u8         log_max_klm_list_size[0x6];

	u8         non_wire_sq[0x1];
	u8         ats_ro_dependence[0x1];
	u8         qpc_extension[0x1];
	u8         log_max_static_sq_wq_size[0x5];
	u8         resources_on_virtio_net_emulation_manager[0x1];
	u8         resources_on_virtio_blk_emulation_manager[0x1];
	u8         log_max_ra_req_dc[0x6];
	u8         vhca_trust_level_reg[0x1];
	u8         eth_wqe_too_small_mode[0x1];
	u8         vnic_env_eth_wqe_too_small[0x1];
	u8         log_max_static_sq_wq[0x5];
	u8         ooo_sl_mask[0x1];
	u8         vnic_env_cq_overrun[0x1];
	u8         log_max_ra_res_dc[0x6];

	u8         cc_roce_ecn_rp_classify_mode[0x1];
	u8         cc_roce_ecn_rp_dynamic_rtt[0x1];
	u8         cc_roce_ecn_rp_dynamic_ai[0x1];
	u8         cc_roce_ecn_rp_dynamic_g[0x1];
	u8         cc_roce_ecn_rp_burst_decouple[0x1];
	u8         release_all_pages[0x1];
	u8         reserved_at_146[0x1];
	u8         sig_crc64_xp10[0x1];
	u8         sig_crc32c[0x1];
	u8         roce_accl[0x1];
	u8         log_max_ra_req_qp[0x6];
	u8         rtr2rts_qp_counters_set_id[0x1];
	u8         rts2rts_udp_sport[0x1];
	u8         rts2rts_lag_tx_port_affinity[0x1];
	u8         dma_mmo_sq[0x1];
	u8         compress_min_block_size[0x4];
	u8         compress_mmo_sq[0x1];
	u8         decompress_mmo_sq[0x1];
	u8         log_max_ra_res_qp[0x6];

	u8         end_pad[0x1];
	u8         cc_query_allowed[0x1];
	u8         cc_modify_allowed[0x1];
	u8         start_pad[0x1];
	u8         cache_line_128byte[0x1];
	u8         gid_table_size_ro[0x1];
	u8         pkey_table_size_ro[0x1];
	u8         rts2rts_qp_rmp[0x1];
	u8         rnr_nak_q_counters[0x1];
	u8         rts2rts_qp_counters_set_id[0x1];
	u8         rts2rts_qp_dscp[0x1];
	u8         gen3_cc_negotiation[0x1];
	u8         vnic_env_int_rq_oob[0x1];
	u8         sbcam_reg[0x1];
	u8         cwcam_reg[0x1];
	u8         qcam_reg[0x1];
	u8         gid_table_size[0x10];

	u8         out_of_seq_cnt[0x1];
	u8         vport_counters[0x1];
	u8         retransmission_q_counters[0x1];
	u8         debug[0x1];
	u8         modify_rq_counters_set_id[0x1];
	u8         rq_delay_drop[0x1];
	u8         max_qp_cnt[0xa];
	u8         pkey_table_size[0x10];

	u8         vport_group_manager[0x1];
	u8         vhca_group_manager[0x1];
	u8         ib_virt[0x1];
	u8         eth_virt[0x1];
	u8         vnic_env_queue_counters[0x1];
	u8         ets[0x1];
	u8         nic_flow_table[0x1];
	u8         eswitch_manager[0x1];
	u8         device_memory[0x1];
	u8         mcam_reg[0x1];
	u8         pcam_reg[0x1];
	u8         local_ca_ack_delay[0x5];
	u8         port_module_event[0x1];
	u8         enhanced_retransmission_q_counters[0x1];
	u8         port_checks[0x1];
	u8         pulse_gen_control[0x1];
	u8         disable_link_up_by_init_hca[0x1];
	u8         beacon_led[0x1];
	u8         port_type[0x2];
	u8         num_ports[0x8];

	u8         reserved_at_1c0[0x1];
	u8         pps[0x1];
	u8         pps_modify[0x1];
	u8         log_max_msg[0x5];
	u8         multi_path_xrc_rdma[0x1];
	u8         multi_path_dc_rdma[0x1];
	u8         multi_path_rc_rdma[0x1];
	u8         traffic_fast_control[0x1];
	u8         max_tc[0x4];
	u8         temp_warn_event[0x1];
	u8         dcbx[0x1];
	u8         general_notification_event[0x1];
	u8         multi_prio_sq[0x1];
	u8         afu_owner[0x1];
	u8         fpga[0x1];
	u8         rol_s[0x1];
	u8         rol_g[0x1];
	u8         ib_port_sniffer[0x1];
	u8         wol_s[0x1];
	u8         wol_g[0x1];
	u8         wol_a[0x1];
	u8         wol_b[0x1];
	u8         wol_m[0x1];
	u8         wol_u[0x1];
	u8         wol_p[0x1];

	u8         stat_rate_support[0x10];
	u8         sig_block_4048[0x1];
	u8         pci_sync_for_fw_update_event[0x1];
	u8         init2rtr_drain_sigerr[0x1];
	u8         log_max_extended_rnr_retry[0x5];
	u8         init2_lag_tx_port_affinity[0x1];
	u8         flow_group_type_hash_split[0x1];
	u8         stat_rate_200[0x1];
	u8         wqe_based_flow_table_update_cap[0x1];
	u8         cqe_version[0x4];

	u8         compact_address_vector[0x1];
	u8         eth_striding_wq[0x1];
	u8         reserved_at_202[0x1];
	u8         ipoib_enhanced_offloads[0x1];
	u8         ipoib_basic_offloads[0x1];
	u8         ib_link_list_striding_wq[0x1];
	u8         repeated_block_disabled[0x1];
	u8         umr_modify_entity_size_disabled[0x1];
	u8         umr_modify_atomic_disabled[0x1];
	u8         umr_indirect_mkey_disabled[0x1];
	u8         umr_fence[0x2];
	u8         dc_req_sctr_data_cqe[0x1];
	u8         dc_connect_qp[0x1];
	u8         dc_cnak_trace[0x1];
	u8         drain_sigerr[0x1];
	u8         cmdif_checksum[0x2];
	u8         sigerr_cqe[0x1];
	u8         e_psv[0x1];
	u8         wq_signature[0x1];
	u8         sctr_data_cqe[0x1];
	u8         bsf_in_create_mkey[0x1];
	u8         sho[0x1];
	u8         tph[0x1];
	u8         rf[0x1];
	u8         dct[0x1];
	u8         qos[0x1];
	u8         eth_net_offloads[0x1];
	u8         roce[0x1];
	u8         atomic[0x1];
	u8         extended_retry_count[0x1];

	u8         cq_oi[0x1];
	u8         cq_resize[0x1];
	u8         cq_moderation[0x1];
	u8         cq_period_mode_modify[0x1];
	u8         cq_invalidate[0x1];
	u8         ib_striding_wq_cq_first_indication[0x1];
	u8         cq_eq_remap[0x1];
	u8         pg[0x1];
	u8         block_lb_mc[0x1];
	u8         exponential_backoff[0x1];
	u8         scqe_break_moderation[0x1];
	u8         cq_period_start_from_cqe[0x1];
	u8         cd[0x1];
	u8         atm[0x1];
	u8         apm[0x1];
	u8         vector_calc[0x1];
	u8         umr_ptr_rlkey[0x1];
	u8         imaicl[0x1];
	u8         qp_packet_based[0x1];
	u8         ib_cyclic_striding_wq[0x1];
	u8         ipoib_enhanced_pkey_change[0x1];
	u8         initiator_src_dct_in_cqe[0x1];
	u8         qkv[0x1];
	u8         pkv[0x1];
	u8         set_deth_sqpn[0x1];
	u8         rts2rts_primary_sl[0x1];
	u8         initiator_src_dct[0x1];
	u8         dc_v2[0x1];
	u8         xrc[0x1];
	u8         ud[0x1];
	u8         uc[0x1];
	u8         rc[0x1];

	u8         uar_4k[0x1];
	u8         dci_no_rdma_wr_optimized_performance[0x1];
	u8         local_dma[0x1];
	u8         log_max_local_dma_size[0x5];
	u8         fl_rc_qp_when_roce_disabled[0x1];
	u8         regexp_params[0x1];
	u8         uar_sz[0x6];
	u8         port_selection_cap[0x1];
	u8         nic_cap_reg[0x1];
	u8         umem_uid_0[0x1];
	u8         log_max_dc_cnak_qps[0x5];
	u8         log_pg_sz[0x8];

	u8         bf[0x1];
	u8         driver_version[0x1];
	u8         pad_tx_eth_packet[0x1];
	u8         query_driver_version[0x1];
	u8         max_qp_retry_freq[0x1];
	u8         qp_by_name[0x1];
	u8         mkey_by_name[0x1];
	u8         virtio_fs_device_emulation_manager[0x1];
	u8         suspend_qp_uc[0x1];
	u8         suspend_qp_ud[0x1];
	u8         suspend_qp_rc[0x1];
	u8         log_bf_reg_size[0x5];
	u8         reserved_at_270[0x1];
	u8         ec_offload_engine_disabled[0x1];
	u8         reserved_at_272[0x1];
	u8         qp_error_syndrome[0x1];
	u8         qp_handling_when_roce_disabled[0x1];
	u8         atomic_rate_pa[0x1];
	u8         lag_dct[0x2];
	u8         lag_tx_port_affinity[0x1];
	u8         lag_native_fdb_selection[0x1];
	u8         reserved_at_27a[0x1];
	u8         lag_master[0x1];
	u8         num_lag_ports[0x4];

	u8         num_of_diagnostic_counters[0x10];
	u8         max_wqe_sz_sq[0x10];

	u8         reserved_at_2a0[0xc];
	u8         regexp_mmo_sq[0x1];
	u8         regexp_version[0x3];
	u8         max_wqe_sz_rq[0x10];

	u8         max_flow_counter_31_16[0x10];
	u8         max_wqe_sz_sq_dc[0x10];

	u8         reserved_at_2e0[0x7];
	u8         max_qp_mcg[0x19];

	u8         mlnx_tag_ethertype[0x10];
	u8         flow_counter_bulk_alloc[0x8];
	u8         log_max_mcg[0x8];

	u8         emulated_vf_exist[0x1];
	u8         uncorrectable_notification_dpu_reset_en[0x1];
	u8         always_fire_port_state_change_event[0x1];
	u8         log_max_transport_domain[0x5];
	u8         tir_esw_lb_filter_disable[0x1];
	u8         flow_table_freeze[0x1];
	u8         reserved_at_32a[0x1];
	u8         log_max_pd[0x5];
	u8         reserved_at_330[0xb];
	u8         log_max_xrcd[0x5];

	u8         nic_receive_steering_discard[0x1];
	u8         receive_discard_vport_down[0x1];
	u8         transmit_discard_vport_down[0x1];
	u8         eq_overrun_count[0x1];
	u8         nic_receive_steering_depth[0x1];
	u8         invalid_command_count[0x1];
	u8         quota_exceeded_count[0x1];
	u8         reserved_at_347[0x1];
	u8         log_max_flow_counter_bulk[0x8];
	u8         max_flow_counter_15_0[0x10];

	u8         modify_tis[0x1];
	u8         flow_counters_dump[0x1];
	u8         pad_tls_encrypt_sig[0x1];
	u8         log_max_rq[0x5];
	u8         reserved_at_368[0x3];
	u8         log_max_sq[0x5];
	u8         reserved_at_370[0x3];
	u8         log_max_tir[0x5];
	u8         reserved_at_378[0x3];
	u8         log_max_tis[0x5];

	u8         basic_cyclic_rcv_wqe[0x1];
	u8         eth_wqe_too_small_discard_no_cqe[0x1];
	u8         mem_rq_rmp[0x1];
	u8         log_max_rmp[0x5];
	u8         reserved_at_388[0x3];
	u8         log_max_rqt[0x5];
	u8         reserved_at_390[0x3];
	u8         log_max_rqt_size[0x5];
	u8         reserved_at_398[0x3];
	u8         log_max_tis_per_sq[0x5];

	u8         ext_stride_num_range[0x1];
	u8         roce_rw_supported[0x1];
	u8         log_max_current_uc_list_wr_supported[0x1];
	u8         log_max_stride_sz_rq[0x5];
	u8         reserved_at_3a8[0x3];
	u8         log_min_stride_sz_rq[0x5];
	u8         reserved_at_3b0[0x3];
	u8         log_max_stride_sz_sq[0x5];
	u8         reserved_at_3b8[0x3];
	u8         log_min_stride_sz_sq[0x5];

	u8         hairpin[0x1];
	u8         reserved_at_3c1[0x2];
	u8         log_max_hairpin_queues[0x5];
	u8         hairpin_ib_raw[0x1];
	u8         hairpin_eth2ipoib[0x1];
	u8         hairpin_ipoib2eth[0x1];
	u8         log_max_hairpin_wq_data_sz[0x5];
	u8         reserved_at_3d0[0x3];
	u8         log_max_hairpin_num_packets[0x5];
	u8         reserved_at_3d8[0x3];
	u8         log_max_wq_sz[0x5];

	u8         nic_vport_change_event[0x1];
	u8         disable_local_lb_uc[0x1];
	u8         disable_local_lb_mc[0x1];
	u8         log_min_hairpin_wq_data_sz[0x5];
	u8         system_image_guid_modifiable[0x1];
	u8         reserved_at_3e9[0x1];
	u8         vhca_state[0x1];
	u8         log_max_vlan_list[0x5];
	u8         reserved_at_3f0[0x1];
	u8         aes_xts_single_block_le_tweak[0x1];
	u8         aes_xts_multi_block_be_tweak[0x1];
	u8         log_max_current_mc_list[0x5];
	u8         reserved_at_3f8[0x3];
	u8         log_max_current_uc_list[0x5];

	u8         general_obj_types_63_0[0x40];

	u8         sq_ts_format[0x2];
	u8         rq_ts_format[0x2];
	u8         steering_format_version[0x4];
	u8         create_qp_start_hint[0x18];

	u8         reserved_at_460[0x1];
	u8         ats[0x1];
	u8         reserved_at_462[0x1];
	u8         log_max_uctx[0x5];
	u8         reserved_at_468[0x1];
	u8         crypto[0x1];
	u8         ipsec_offload[0x1];
	u8         log_max_umem[0x5];
	u8         max_num_eqs[0x10];

	u8         sigerr_domain_and_sig_type[0x1];
	u8         tls_tx[0x1];
	u8         tls_rx[0x1];
	u8         log_max_l2_table[0x5];
	u8         reserved_at_488[0x8];
	u8         log_uar_page_sz[0x10];

	u8         e[0x1];
	u8         reserved_at_4a1[0x1f];

	u8         device_frequency_mhz[0x20];

	u8         device_frequency_khz[0x20];

	u8         capi[0x1];
	u8         create_pec[0x1];
	u8         nvmf_target_offload[0x1];
	u8         capi_invalidate[0x1];
	u8         reserved_at_504[0x17];
	u8         log_max_pasid[0x5];

	u8         num_of_uars_per_page[0x20];

	u8         flex_parser_protocols[0x20];

	u8         max_geneve_tlv_options[0x8];
	u8         reserved_at_568[0x3];
	u8         max_geneve_tlv_option_data_len[0x5];
	u8         flex_parser_header_modify[0x1];
	u8         reserved_at_571[0x2];
	u8         log_max_guaranteed_connections[0x5];
	u8         driver_version_before_init_hca[0x1];
	u8         adv_virtualization[0x1];
	u8         reserved_at_57a[0x1];
	u8         log_max_dct_connections[0x5];

	u8         log_max_atomic_size_qp[0x8];
	u8         reserved_at_588[0x3];
	u8         log_max_dci_stream_channels[0x5];
	u8         reserved_at_590[0x3];
	u8         log_max_dci_errored_streams[0x5];
	u8         log_max_atomic_size_dc[0x8];

	u8         max_multi_user_group_size[0x10];
	u8         enhanced_cqe_compression[0x1];
	u8         reserved_at_5b1[0x1];
	u8         crossing_vhca_mkey[0x1];
	u8         log_max_dek[0x5];
	u8         reserved_at_5b8[0x1];
	u8         mini_cqe_resp_l3_l4_tag[0x1];
	u8         mini_cqe_resp_flow_tag[0x1];
	u8         reserved_at_5bb[0x1];
	u8         mini_cqe_resp_stride_index[0x1];
	u8         cqe_128_always[0x1];
	u8         cqe_compression_128b[0x1];
	u8         cqe_compression[0x1];

	u8         cqe_compression_timeout[0x10];
	u8         cqe_compression_max_num[0x10];

	u8         must_not_use[0x8];
	u8         flex_parser_id_gtpu_dw_0[0x4];
	u8         log_max_tm_offloaded_op_size[0x4];
	u8         tag_matching[0x1];
	u8         rndv_offload_rc[0x1];
	u8         rndv_offload_dc[0x1];
	u8         log_tag_matching_list_sz[0x5];
	u8         modify_xrq[0x1];
	u8         sync_steering_cache_inv[0x1];
	u8         must_not_use2[0x1];
	u8         log_max_xrq[0x5];

	u8         affiliate_nic_vport_criteria[0x8];
	u8         native_port_num[0x8];
	u8         num_vhca_ports[0x8];
	u8         flex_parser_id_gtpu_teid[0x4];
	u8         reserved_at_61c[0x1];
	u8         trusted_vnic_vhca[0x1];
	u8         sw_owner_id[0x1];
	u8         reserve_not_to_use[0x1];

	u8         max_num_of_monitor_counters[0x10];
	u8         num_ppcnt_monitor_counters[0x10];

	u8         max_num_sf[0x10];
	u8         num_q_monitor_counters[0x10];

	u8         reserved_at_660[0x20];

	u8         sf[0x1];
	u8         sf_set_partition[0x1];
	u8         reserved_at_682[0x1];
	u8         log_max_sf[0x5];
	u8         dpa[0x1];
	u8         reserved_at_689[0x2];
	u8         wait_on_data[0x1];
	u8         wait_on_time[0x1];
	u8         migration[0x1];
	u8         cq_time_stamp_real_time_mode[0x2];
	u8         log_min_sf_size[0x8];
	u8         max_num_sf_partitions[0x8];

	u8         uctx_permission[0x20];

	u8         flex_parser_id_mpls_over_x_cw[0x4];
	u8         flex_parser_id_geneve_tlv_option_0[0x4];
	u8         flex_parser_id_icmp_dw1[0x4];
	u8         flex_parser_id_icmp_dw0[0x4];
	u8         flex_parser_id_icmpv6_dw1[0x4];
	u8         flex_parser_id_icmpv6_dw0[0x4];
	u8         flex_parser_id_outer_first_mpls_over_gre[0x4];
	u8         flex_parser_id_outer_first_mpls_over_udp_label[0x4];

	u8         max_num_match_definer[0x10];
	u8         sf_base_id[0x10];

	u8         flex_parser_id_gtpu_dw_2[0x4];
	u8         flex_parser_id_gtpu_first_ext_dw_0[0x4];
	u8         num_total_dynamic_vf_msix[0x18];

	u8         reserved_at_720[0x3];
	u8         log_flow_hit_aso_granularity[0x5];
	u8         reserved_at_728[0x3];
	u8         log_flow_hit_aso_max_alloc[0x5];
	u8         reserved_at_730[0x4];
	u8         dynamic_msix_table_size[0xc];

	u8         sha_mmo_qp[0x1];
	u8         reserved_at_741[0x2];
	u8         log_max_num_flow_hit_aso[0x5];
	u8         dma_mmo_qp[0x1];
	u8         regexp_mmo_qp[0x1];
	u8         compress_mmo_qp[0x1];
	u8         decompress_mmo_qp[0x1];
	u8         min_dynamic_vf_msix_table_size[0x4];
	u8         reserved_at_750[0x4];
	u8         max_dynamic_vf_msix_table_size[0xc];

	u8         reserved_at_760[0x3];
	u8         log_max_num_header_modify_argument[0x5];
	u8         log_header_modify_argument_granularity_offset[0x4];
	u8         log_header_modify_argument_granularity[0x4];
	u8         reserved_at_770[0x3];
	u8         log_header_modify_argument_max_alloc[0x5];
	u8         reserved_at_778[0x3];
	u8         max_flow_execute_aso[0x5];

	u8         vhca_tunnel_commands[0x40];

	u8         match_definer_format_supported[0x40];
};

struct mlx5_ifc_cmd_hca_cap_2_bits {
	u8         reserved_at_0[0x80];

	u8         migratable[0x1];
	u8         force_multi_prio_sq[0x1];
	u8         cq_with_emulated_dev_eq[0x1];
	u8         max_num_prog_sample_field[0x5];
	u8         multi_path_force[0x1];
	u8         fw_cpu_monitoring[0x1];
	u8         enh_eth_striding_wq[0x1];
	u8         log_max_num_reserved_qpn[0x5];
	u8         reserved_at_90[0x1];
	u8         introspection_mkey_access_allowed[0x1];
	u8         query_vuid[0x1];
	u8         log_reserved_qpn_granularity[0x5];
	u8         reserved_at_98[0x3];
	u8         log_reserved_qpn_max_alloc[0x5];

	u8         max_reformat_insert_size[0x8];
	u8         max_reformat_insert_offset[0x8];
	u8         max_reformat_remove_size[0x8];
	u8         max_reformat_remove_offset[0x8];

	u8         multi_sl_qp[0x1];
	u8         non_tunnel_reformat[0x1];
	u8         bsf_v2[0x1];
	u8         log_min_stride_wqe_sz[0x5];
	u8         migration_multi_load[0x1];
	u8         migration_tracking_state[0x1];
	u8         reserved_at_ca[0x1];
	u8         log_conn_track_granularity[0x5];
	u8         reserved_at_d0[0x3];
	u8         log_conn_track_max_alloc[0x5];
	u8         reserved_at_d8[0x3];
	u8         log_max_conn_track_offload[0x5];

	u8         cross_vhca_object_to_object_supported[0x20];

	u8         allowed_object_for_other_vhca_access[0x40];

	u8         introspection_mkey[0x20];

	u8         ec_mmo_qp[0x1];
	u8         sync_driver_version[0x1];
	u8         driver_version_change_event[0x1];
	u8         hairpin_sq_wqe_bb_size[0x5];
	u8         hairpin_sq_wq_in_host_mem[0x1];
	u8         hairpin_data_buffer_locked[0x1];
	u8         reserved_at_16a[0x1];
	u8         log_ec_mmo_max_size[0x5];
	u8         reserved_at_170[0x3];
	u8         log_ec_mmo_max_src[0x5];
	u8         reserved_at_178[0x3];
	u8         log_ec_mmo_max_dst[0x5];

	u8         sync_driver_actions[0x20];

	u8         flow_table_type_2_type[0x8];
	u8         reserved_at_1a8[0x2];
	u8         format_select_dw_8_6_ext[0x1];
	u8         log_min_mkey_entity_size[0x5];
	u8         execute_aso_type[0x10];

	u8         general_obj_types_127_64[0x40];

	u8         repeated_mkey_v2[0x20];

	u8         reserved_gid_index_valid[0x1];
	u8         sw_vhca_id_valid[0x1];
	u8         sw_vhca_id[0xe];
	u8         reserved_gid_index[0x10];

	u8         reserved_at_240[0x3];
	u8         log_max_channel_service_connection[0x5];
	u8         reserved_at_248[0x3];
	u8         ts_cqe_metadata_size2wqe_counter[0x5];
	u8         reserved_at_250[0x3];
	u8         flow_counter_bulk_log_max_alloc[0x5];
	u8         reserved_at_258[0x3];
	u8         flow_counter_bulk_log_granularity[0x5];

	u8         format_select_dw_mpls_over_x_cw[0x8];
	u8         format_select_dw_geneve_tlv_option_0[0x8];
	u8         format_select_dw_outer_first_mpls_over_gre[0x8];
	u8         format_select_dw_outer_first_mpls_over_udp[0x8];

	u8         format_select_dw_gtpu_dw_0[0x8];
	u8         format_select_dw_gtpu_dw_1[0x8];
	u8         format_select_dw_gtpu_dw_2[0x8];
	u8         format_select_dw_gtpu_first_ext_dw_0[0x8];

	u8         generate_wqe_type[0x20];

	u8         max_enh_strwq_supported_profile[0x10];
	u8         reserved_at_2d0[0x3];
	u8         log_max_total_hairpin_data_buffer_locked_size[0x5];
	u8         reserved_at_2d8[0x3];
	u8         log_max_rq_hairpin_data_buffer_locked_size[0x5];

	u8         reserved_at_2e0[0xb];
	u8         enh_strwq_max_log_page_size[0x5];
	u8         enh_strwq_max_headroom[0x8];
	u8         enh_strwq_max_tailroom[0x8];

	u8         migration_tag_version_0[4][0x20];

	u8         reserved_at_380[0x480];
};

struct mlx5_ifc_roce_cap_bits {
	u8         roce_apm[0x1];
	u8         rts2rts_primary_eth_prio[0x1];
	u8         roce_rx_allow_untagged[0x1];
	u8         rts2rts_src_addr_index_for_vlan_valid_vlan_id[0x1];
	u8         sw_r_roce_src_udp_port[0x1];
	u8         fl_rc_qp_when_roce_disabled[0x1];
	u8         fl_rc_qp_when_roce_enabled[0x1];
	u8         reserved_at_7[0x17];
	u8         qp_ts_format[0x2];

	u8         reserved_at_20[0x60];

	u8         reserved_at_80[0xc];
	u8         l3_type[0x4];
	u8         reserved_at_90[0x8];
	u8         roce_version[0x8];

	u8         reserved_at_a0[0x10];
	u8         r_roce_dest_udp_port[0x10];

	u8         r_roce_max_src_udp_port[0x10];
	u8         r_roce_min_src_udp_port[0x10];

	u8         reserved_at_e0[0x10];
	u8         roce_address_table_size[0x10];

	u8         reserved_at_100[0x700];
};

union mlx5_ifc_hca_cap_union_bits {
	struct mlx5_ifc_cmd_hca_cap_bits cmd_hca_cap;
	struct mlx5_ifc_cmd_hca_cap_2_bits cmd_hca_cap_2;
	struct mlx5_ifc_roce_cap_bits cmd_hca_cap_roce;
	u8         reserved_at_0[0x8000];
};

struct mlx5_ifc_query_hca_cap_out_bits {
	u8         status[0x8];
	u8         reserved_at_8[0x18];

	u8         syndrome[0x20];

	u8         reserved_at_40[0x40];

	union mlx5_ifc_hca_cap_union_bits capability;
};

enum {
	MLX5_HCA_CAP_OP_MOD_GENERAL_DEVICE        = 0x0 << 1,
	MLX5_HCA_CAP_OP_MOD_ETHERNET_OFFLOAD      = 0x1 << 1,
	MLX5_HCA_CAP_OP_MOD_ROCE                  = 0x4 << 1,
	MLX5_HCA_CAP_OP_MOD_NIC_FLOW_TABLE        = 0x7 << 1,
	MLX5_HCA_CAP_OP_MOD_ESW_FLOW_TABLE        = 0x8 << 1,
	MLX5_HCA_CAP_OP_MOD_QOS                   = 0xc << 1,
	MLX5_HCA_CAP_OP_MOD_ESW                   = 0x9 << 1,
	MLX5_HCA_CAP_OP_MOD_DEVICE_MEMORY         = 0xf << 1,
	MLX5_HCA_CAP_OP_MOD_IPSEC                 = 0x15 << 1,
	MLX5_HCA_CAP_OP_MOD_CRYPTO                = 0x1a << 1,
	MLX5_HCA_CAP_OP_MOD_GENERAL_DEVICE_CAP_2  = 0x20 << 1,
};

enum mlx5_cap_mode {
	MLX5_HCA_CAP_OPMOD_GET_MAX = 0,
	MLX5_HCA_CAP_OPMOD_GET_CUR = 1,
};

struct mlx5_ifc_query_hca_cap_in_bits {
	u8         opcode[0x10];
	u8         reserved_at_10[0x10];

	u8         reserved_at_20[0x10];
	u8         op_mod[0x10];

	u8         other_function[0x1];
	u8         reserved_at_41[0xf];
	u8         function_id[0x10];

	u8         reserved_at_60[0x20];
};

/* PRM section 8.14 Receive Queue - Work Queue (WQ) Format */

enum {
	MLX5_WQ_TYPE_LINKED_LIST = 0x0,
	MLX5_WQ_TYPE_CYCLIC = 0x1,
	MLX5_WQ_TYPE_STRQ_LINKED_LIST = 0x2,
	MLX5_WQ_TYPE_STRQ_CYCLIC = 0x3,
};

enum {
	MLX5_QPC_RQ_TYPE_REGULAR              = 0x0,
	MLX5_QPC_RQ_TYPE_SRQ_RMP_XRC_SRQ_XRQ  = 0x1,
	MLX5_QPC_RQ_TYPE_ZERO_SIZE_RQ         = 0x3,
};

enum {
	MLX5_WQ_END_PAD_MODE_NONE = 0x0,
	MLX5_WQ_END_PAD_MODE_ALIGN = 0x1,
};

struct mlx5_ifc_cmd_pas_bits {
	u8 pa_h[0x20];

	u8 pa_l[0x14];
	u8 reserved_0[0xc];
};

struct mlx5_ifc_wq_bits {
	u8 wq_type[0x4];
	u8 wq_signature[0x1];
	u8 end_padding_mode[0x2];
	u8 cd_slave[0x1];
	u8 reserved_at_8[0x18];

	u8 hds_skip_first_sge[0x1];
	u8 log2_hds_buf_size[0x3];
	u8 reserved_at_24[0x7];
	u8 page_offset[0x5];
	u8 lwm[0x10];

	u8 reserved_at_40[0x8];
	u8 pd[0x18];

	u8 reserved_at_60[0x8];
	u8 uar_page[0x18];

	u8 dbr_addr[0x40];

	u8 hw_counter[0x20];

	u8 sw_counter[0x20];

	u8 reserved_at_100[0xc];
	u8 log_wq_stride[0x4];
	u8 reserved_at_110[0x3];
	u8 log_wq_pg_sz[0x5];
	u8 reserved_at_118[0x3];
	u8 log_wq_sz[0x5];

	u8 dbr_umem_valid[0x1];
	u8 wq_umem_valid[0x1];
	u8 reserved_at_122[0x1];
	u8 log_hairpin_num_packets[0x5];
	u8 reserved_at_128[0x3];
	u8 log_hairpin_data_sz[0x5];
	u8 reserved_at_130[0x4];
	u8 single_wqe_log_num_of_strides[0x4];
	u8 two_byte_shift_en[0x1];
	u8 reserved_at_139[0x4];
	u8 single_stride_log_num_of_bytes[0x3];

	u8 dbr_umem_id[0x20];

	u8 wq_umem_id[0x20];

	u8 wq_umem_offset[0x40];

	u8 reserved_at_1c0[0x440];

	struct mlx5_ifc_cmd_pas_bits pas[0];
};


/* PRM section 29.24.20 QUERY_VUID - Query Vendor Unique Identifier.*/

struct mlx5_ifc_vuid_bits {
	u8         vuid[0x400];
};

struct mlx5_ifc_query_vuid_out_bits {
	u8         status[0x8];
	u8         reserved_at_8[0x18];

	u8         syndrome[0x20];

	u8         reserved_at_40[0x1a0];

	u8         reserved_at_1e0[0x10];
	u8         num_of_entries[0x10];

	struct mlx5_ifc_vuid_bits vuids[0];
};

struct mlx5_ifc_query_vuid_in_bits {
	u8         opcode[0x10];
	u8         reserved_at_10[0x10];

	u8         reserved_at_20[0x10];
	u8         op_mod[0x10];

	u8         reserved_at_40[0x20];

	u8         query_vfs_vuid[0x1];
	u8         reserved_at_61[0xf];
	u8         vhca_id[0x10];
};

/* PRM section 9.6.1 Mkey Context*/

enum {
	MLX5_MKC_ACCESS_MODE_PA    = 0x0,
	MLX5_MKC_ACCESS_MODE_MTT   = 0x1,
	MLX5_MKC_ACCESS_MODE_KLMS  = 0x2,
	MLX5_MKC_ACCESS_MODE_KSM   = 0x3,
	MLX5_MKC_ACCESS_MODE_SW_ICM = 0x4,
	MLX5_MKC_ACCESS_MODE_MEMIC = 0x5,
	MLX5_MKC_ACCESS_MODE_CROSSING_VHCA_MKEY = 0x6,
};

struct mlx5_ifc_mkc_bits {
	u8         reserved_at_0[0x1];
	u8         free[0x1];
	u8         reserved_at_2[0x1];
	u8         access_mode_4_2[0x3];
	u8         reserved_at_6[0x7];
	u8         relaxed_ordering_write[0x1];
	u8         reserved_at_e[0x1];
	u8         small_fence_on_rdma_read_response[0x1];
	u8         umr_en[0x1];
	u8         a[0x1];
	u8         rw[0x1];
	u8         rr[0x1];
	u8         lw[0x1];
	u8         lr[0x1];
	u8         access_mode_1_0[0x2];
	u8         reserved_at_18[0x1];
	u8         tunneled_atomic[0x1];
	u8         ma_translation_mode[0x2];
	u8         reserved_at_1c[0x4];

	u8         qpn[0x18];
	u8         mkey_7_0[0x8];

	u8         reserved_at_40[0x8];
	u8         pasid[0x18];

	u8         length64[0x1];
	u8         bsf_en[0x1];
	u8         sync_umr[0x1];
	u8         reserved_at_63[0x2];
	u8         expected_sigerr_count[0x1];
	u8         reserved_at_66[0x1];
	u8         en_rinval[0x1];
	u8         pd[0x18];

	u8         start_addr[0x40];

	u8         len[0x40];

	u8         bsf_octword_size[0x20];

	u8         reserved_at_120[0x60];

	u8         crossing_target_vhca_id[0x10];
	u8         reserved_at_190[0x10];

	u8         translations_octword_size_crossing_target_mkey[0x20];

	u8         reserved_at_1c0[0x19];
	u8         relaxed_ordering_read[0x1];
	u8         reserved_at_1d9[0x1];
	u8         log_entity_size[0x5];

	u8         reserved_at_1e0[0x20];
};

/* PRM section 29.7.1 Create Mkey */

struct mlx5_ifc_create_mkey_out_bits {
	u8         status[0x8];
	u8         reserved_at_8[0x18];

	u8         syndrome[0x20];

	u8         reserved_at_40[0x8];
	u8         mkey_index[0x18];

	u8         reserved_at_60[0x20];
};

struct mlx5_ifc_create_mkey_in_bits {
	u8         opcode[0x10];
	u8         uid[0x10];

	u8         reserved_at_20[0x10];
	u8         op_mod[0x10];

	u8         reserved_at_40[0x8];
	u8         input_mkey_index[0x18];

	u8         pg_access[0x1];
	u8         mkey_umem_valid[0x1];
	u8         reserved_at_62[0x1e];

	struct mlx5_ifc_mkc_bits memory_key_mkey_entry;

	u8         reserved_at_280[0x80];

	u8         translations_octword_actual_size[0x20];

	u8         mkey_umem_id[0x20];

	u8         mkey_umem_offset[0x40];

	u8         reserved_at_380[0x500];

	u8         klm_pas_mtt[][0x20];
};

struct mlx5_ifc_klm_bits {
	uint8_t byte_count[0x20];
	uint8_t mkey[0x20];
	uint8_t address[0x40];
};

/* PRM section 29.32.10 Allow Other Vhca Access */

struct mlx5_ifc_allow_other_vhca_access_in_bits {
	u8         opcode[0x10];
	u8         uid[0x10];

	u8         reserved_at_20[0x10];
	u8         op_mod[0x10];

	u8         reserved_at_40[0x40];

	u8         reserved_at_80[0x10];
	u8         object_type_to_be_accessed[0x10];

	u8         object_id_to_be_accessed[0x20];

	u8         reserved_at_c0[0x40];

	u8         access_key[0x100];
};

struct mlx5_ifc_allow_other_vhca_access_out_bits {
	u8         status[0x8];
	u8         reserved_at_8[0x18];

	u8         syndrome[0x20];

	u8         reserved_at_40[0x40];
};

/* PRM section 29.24.1 Create General Obj */

struct mlx5_ifc_general_obj_in_cmd_hdr_bits {
	u8         opcode[0x10];
	u8         uid[0x10];

	u8         vhca_tunnel_id[0x10];
	u8         obj_type[0x10];

	u8         obj_id[0x20];

	u8         alias_object[0x1];
	u8         reserved_at_61[0x2];
	u8         log_obj_range[0x5];
	u8         reserved_at_68[0x18];
};

struct mlx5_ifc_general_obj_out_cmd_hdr_bits {
	u8         status[0x8];
	u8         reserved_at_8[0x18];

	u8         syndrome[0x20];

	u8         obj_id[0x20];

	u8         reserved_at_60[0x20];
};

struct mlx5_ifc_alias_context_bits {
	u8         vhca_id_to_be_accessed[0x10];
	u8         reserved_at_10[0xd];
	u8         status[0x3];

	u8         object_id_to_be_accessed[0x20];

	u8         reserved_at_40[0x40];

	u8         access_key[0x100];

	u8         metadata[0x80];
};

struct mlx5_ifc_create_alias_obj_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_alias_context_bits alias_ctx;
};

struct mlx5_ifc_create_alias_obj_out_bits {
	struct mlx5_ifc_general_obj_out_cmd_hdr_bits hdr;
	struct mlx5_ifc_alias_context_bits alias_ctx;
};

struct mlx5_ifc_alloc_pd_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];

	u8 syndrome[0x20];

	u8 reserved_at_40[0x8];
	u8 pd[0x18];

	u8 reserved_at_60[0x20];
};

struct mlx5_ifc_alloc_pd_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];

	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];

	u8 reserved_at_40[0x40];
};

struct mlx5_ifc_dealloc_pd_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];

	u8 syndrome[0x20];

	u8 reserved_at_40[0x40];
};

struct mlx5_ifc_dealloc_pd_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];

	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];

	u8 reserved_at_40[0x8];
	u8 pd[0x18];

	u8 reserved_at_60[0x20];
};

/* PRM section 29.11.19 QUERY_EMULATED_FUNCTIONS_INFO - Query Emulated Functions Information */

enum mlx5_emulated_functions_op_mod {
        MLX5_SET_EMULATED_FUNCTIONS_OP_MOD_NVME_PHYSICAL_FUNCTIONS = 0x0,
        MLX5_SET_EMULATED_FUNCTIONS_OP_MOD_VIRTIO_NET_PHYSICAL_FUNCTIONS = 0x1,
        MLX5_SET_EMULATED_FUNCTIONS_OP_MOD_VIRTIO_BLK_PHYSICAL_FUNCTIONS = 0x2,
        MLX5_SET_EMULATED_FUNCTIONS_OP_MOD_VIRTUAL_FUNCTIONS = 0x3,
        MLX5_SET_EMULATED_FUNCTIONS_OP_MOD_VIRTIO_FS_PHYSICAL_FUNCTIONS = 0x5,
	MLX5_SET_EMULATED_FUNCTIONS_OP_MOD_INVALID = -1,
};

struct mlx5_ifc_emulated_function_info_bits {
        u8       pci_bdf[0x10];
        u8       vhca_id[0x10];

        u8       hotplug_function[0x1];
        u8       max_num_vfs_valid[0x1];
        u8       vf_exist[0x1];
        u8       reserved_at_22[0xd];
        u8       max_num_vfs[0x10];
};

struct mlx5_ifc_query_emulated_functions_info_out_bits {
        u8       status[0x8];
        u8       reserved_at_8[0x18];

        u8       syndrome[0x20];

        u8       reserved_at_40[0x20];

        u8       reserved_at_60[0x10];
        u8       num_emulated_functions[0x10];

        struct mlx5_ifc_emulated_function_info_bits emulated_function_info[0];
};

struct mlx5_ifc_query_emulated_functions_info_in_bits {
        u8       opcode[0x10];
        u8       uid[0x10];

        u8       reserved_at_20[0x10];
        u8       op_mod[0x10];

        u8       reserved_at_40[0x20];

        u8       reserved_at_60[0x10];
        u8       pf_vhca_id[0x10];
};

/* PRM section 29.24.11 HOTPLUG_DEVICE */

struct mlx5_ifc_device_pci_params_bits {
	uint8_t device_id[0x10];
	uint8_t vendor_id[0x10];

	uint8_t revision_id[0x8];
	uint8_t class_code[0x18];

	uint8_t subsystem_id[0x10];
	uint8_t subsystem_vendor_id[0x10];

	uint8_t num_of_vfs[0x10];
	uint8_t num_msix[0x10];
};

struct mlx5_ifc_nvme_initial_registers_bits {
	uint8_t cap[0x40];
	uint8_t vs[0x20];
	uint8_t intms[0x20];
	uint8_t intmc[0x20];
	uint8_t cc[0x20];
	uint8_t rsvd[0x20];
	uint8_t csts[0x20];
	uint8_t nssr[0x20];
	uint8_t aqa[0x20];
	uint8_t asq[0x40];
	uint8_t acq[0x40];
	uint8_t cmbloc[0x20];
	uint8_t cmbsz[0x20];
	uint8_t bpinfo[0x20];
	uint8_t bprsel[0x20];
	uint8_t bpmbl[0x40];
};

struct mlx5_ifc_virtio_net_config_bits {
	uint8_t mac_47_16[0x20];

	uint8_t mac_15_0[0x10];
	uint8_t status[0x10];

	uint8_t max_virtqueue_pairs[0x10];
	uint8_t mtu[0x10];

	uint8_t reserved_at_60[0x20];
};

struct mlx5_ifc_virtio_net_initial_registers_bits {
	uint8_t device_features[0x40];

	uint8_t reserved_at_40[0x10];
	uint8_t queue_size[0x10];

	uint8_t reserved_at_60[0x20];

	struct mlx5_ifc_virtio_net_config_bits virtio_net_config;
};

struct mlx5_ifc_virtio_blk_config_bits {
	uint8_t capacity[0x40];

	uint8_t size_max[0x20];

	uint8_t seg_max[0x20];

	uint8_t reserved_at_80[0x20];

	uint8_t cylinders[0x10];
	uint8_t heads[0x8];
	uint8_t sectors[0x8];

	uint8_t reserved_at_c0[0x20];

	uint8_t blk_size[0x20];

	uint8_t reserved_at_100[0x20];

	uint8_t physical_blk_exp[0x8];
	uint8_t alignment_offset[0x8];
	uint8_t min_io_size[0x10];

	uint8_t opt_io_size[0x20];

	uint8_t reserved_at_140[0x20];

	uint8_t writeback[0x8];
	uint8_t reserved_at_168[0x8];
	uint8_t num_queues[0x10];

	uint8_t max_discard_sectors[0x20];

	uint8_t max_discard_seg[0x20];

	uint8_t discard_sector_alignment[0x20];

	uint8_t max_write_zeroes_sectors[0x20];

	uint8_t max_write_zeroes_segs[0x20];

	uint8_t write_zeroes_may_unmap[0x8];
	uint8_t reserved_at_228[0x18];

	uint8_t reserved_at_240[0x20];
};

struct mlx5_ifc_virtio_blk_initial_registers_bits {
	uint8_t device_features[0x40];

	uint8_t num_queues[0x10];
	uint8_t queue_size[0x10];

	uint8_t reserved_at_60[0x20];

	struct mlx5_ifc_virtio_blk_config_bits virtio_blk_config;
};

struct mlx5_ifc_virtio_fs_config_bits {
	uint8_t tag[0x120];
	uint8_t num_request_queues[0x20];
	uint8_t reserved_at_140[0x140];
};

struct mlx5_ifc_virtio_fs_initial_registers_bits {
	uint8_t device_features[0x40];

	uint8_t num_queues[0x10];
	uint8_t queue_size[0x10];

	uint8_t reserved_at_60[0x20];

	struct mlx5_ifc_virtio_fs_config_bits virtio_fs_config;
};

union mlx5_ifc_hotplug_device_obj_emulation_initial_registers_auto_bits {
	struct mlx5_ifc_nvme_initial_registers_bits nvme;
	struct mlx5_ifc_virtio_net_initial_registers_bits virtio_net;
	struct mlx5_ifc_virtio_blk_initial_registers_bits virtio_blk;
	struct mlx5_ifc_virtio_fs_initial_registers_bits  virtio_fs;
	uint8_t reserved_at_0[0x600];
};

enum mlx5_hotplug_device_type{
	MLX5_HOTPLUG_DEVICE_TYPE_NVME       = 0x0,
	MLX5_HOTPLUG_DEVICE_TYPE_VIRTIO_NET = 0x1,
	MLX5_HOTPLUG_DEVICE_TYPE_VIRTIO_BLK = 0x2,
	MLX5_HOTPLUG_DEVICE_TYPE_VIRTIO_FS  = 0x3,
};

struct mlx5_ifc_hotplug_device_obj_bits {
	uint8_t modify_field_select[0x40];

	uint8_t reserved_at_40[0x17];
	uint8_t initial_registers_valid[0x1];
	uint8_t device_type[0x8];

	uint8_t total_vf[0x10];
	uint8_t vhca_id[0x10];

	struct mlx5_ifc_device_pci_params_bits pci_params;

	uint8_t reserved_at_e0[0x100];

	union mlx5_ifc_hotplug_device_obj_emulation_initial_registers_auto_bits emulation_initial_regs;
};

struct mlx5_ifc_hotplug_device_in_bits {
	uint8_t opcode[0x10];
	uint8_t uid[0x10];

	uint8_t reserved_at_20[0x20];

	uint8_t reserved_at_40[0x40];

	struct mlx5_ifc_hotplug_device_obj_bits	hotplug_device_object;
};

struct mlx5_ifc_hotplug_device_out_bits {
	uint8_t status[0x8];
	uint8_t reserved_at_8[0x18];

	uint8_t syndrome[0x20];

	uint8_t reserved_at_40[0x40];

	struct mlx5_ifc_hotplug_device_obj_bits	hotplug_device_object;
};

struct mlx5_ifc_hotunplug_device_in_bits {
	uint8_t opcode[0x10];
	uint8_t uid[0x10];

	uint8_t reserved_at_20[0x20];

	uint8_t reserved_at_40[0x40];

	struct mlx5_ifc_hotplug_device_obj_bits	hotplug_device_object;
};

struct mlx5_ifc_hotunplug_device_out_bits {
	uint8_t status[0x8];
	uint8_t reserved_at_8[0x18];

	uint8_t syndrome[0x20];

	uint8_t reserved_at_40[0x40];
};

/* PRM section 8.21 Events and Interrupts */

enum {
	MLX5_EVENT_TYPE_COMPLETION_EVENTS = 0x0,
};

/* PRM section 8.17 Receive Memory Pool (RMP) - RMP Context Format */

enum {
	MLX5_RMPC_STATE_RDY = 0x1,
	MLX5_RMPC_STATE_ERR = 0x3,
};

struct mlx5_ifc_rmpc_bits {
	uint8_t reserved_0[0x8];
	uint8_t state[0x4];
	uint8_t reserved_1[0x14];

	uint8_t basic_cyclic_rcv_wqe[0x1];
	uint8_t reserved_2[0x1f];

	uint8_t reserved_3[0x140];

	struct mlx5_ifc_wq_bits wq;
};

/* PRM section 29.18.1 Create Receive Memory Pool */

struct mlx5_ifc_create_rmp_out_bits {
	uint8_t status[0x8];
	uint8_t reserved_0[0x18];

	uint8_t syndrome[0x20];

	uint8_t reserved_1[0x8];
	uint8_t rmpn[0x18];

	uint8_t reserved_2[0x20];
};

struct mlx5_ifc_create_rmp_in_bits {
	uint8_t opcode[0x10];
	uint8_t reserved_0[0x10];

	uint8_t reserved_1[0x10];
	uint8_t op_mod[0x10];

	uint8_t reserved_2[0xc0];

	struct mlx5_ifc_rmpc_bits ctx;
};

/* PRM section 29.10 CQ Commands */

/* Page size is provided in granularity of 4K. */
#define MLX5_ADAPTER_PAGE_SHIFT 12

enum {
	MLX5_CQE_SIZE_64B = 0x0,
	MLX5_CQE_SIZE_128B = 0x1,
};

struct mlx5_ifc_cqc_bits {
	u8 status[0x4];
	u8 as_notify[0x1];
	u8 initiator_src_dct[0x1];
	u8 dbr_umem_valid[0x1];
	u8 reserved_at_7[0x1];
	u8 cqe_sz[0x3];
	u8 cc[0x1];
	u8 reserved_at_c[0x1];
	u8 scqe_break_moderation_en[0x1];
	u8 oi[0x1];
	u8 cq_period_mode[0x2];
	u8 cqe_comp_en[0x1];
	u8 mini_cqe_res_format[0x2];
	u8 st[0x4];
	u8 reserved_at_18[0x1];
	u8 cqe_comp_layout[0x7];

	u8 dbr_umem_id[0x20];

	u8 reserved_at_40[0x14];
	u8 page_offset[0x6];
	u8 reserved_at_5a[0x2];
	u8 mini_cqe_res_format_ext[0x2];
	u8 cq_timestamp_format[0x2];

	u8 reserved_at_60[0x3];
	u8 log_cq_size[0x5];
	u8 uar_page[0x18];

	u8 reserved_at_80[0x4];
	u8 cq_period[0xc];
	u8 cq_max_count[0x10];

	u8 reserved_at_a0[0x18];
	u8 c_eqn[0x8];

	u8 reserved_at_c0[0x3];
	u8 log_page_size[0x5];
	u8 reserved_at_c8[0x18];

	u8 reserved_at_e0[0x20];

	u8 reserved_at_100[0x8];
	u8 last_notified_index[0x18];

	u8 reserved_at_120[0x8];
	u8 last_solicit_index[0x18];

	u8 reserved_at_140[0x8];
	u8 consumer_counter[0x18];

	u8 reserved_at_160[0x8];
	u8 producer_counter[0x18];

	u8 reserved_at_180[0x40];

	u8 dbr_addr[0x40];
};

struct mlx5_ifc_create_cq_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];
	u8 syndrome[0x20];
	u8 reserved_at_40[0x8];
	u8 cqn[0x18];
	u8 reserved_at_60[0x20];
};

struct mlx5_ifc_create_cq_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];

	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];

	u8 reserved_at_40[0x40];

	struct mlx5_ifc_cqc_bits cq_context;

	u8 cq_umem_offset[0x40];

	u8 cq_umem_id[0x20];

	u8 cq_umem_valid[0x1];
	u8 reserved_at_2e1[0x1f];

	u8 reserved_at_300[0x580];
	u8 pas[];
};

struct mlx5_ifc_ads_bits {
	u8 fl[0x1];
	u8 free_ar[0x1];
	u8 reserved_at_2[0xe];
	u8 pkey_index[0x10];

	u8 reserved_at_20[0x8];
	u8 grh[0x1];
	u8 mlid[0x7];
	u8 rlid[0x10];

	u8 ack_timeout[0x5];
	u8 reserved_at_45[0x3];
	u8 src_addr_index[0x8];
	u8 reserved_at_50[0x4];
	u8 stat_rate[0x4];
	u8 hop_limit[0x8];

	u8 reserved_at_60[0x4];
	u8 tclass[0x8];
	u8 flow_label[0x14];

	u8 rgid_rip[16][0x8];

	u8 reserved_at_100[0x4];
	u8 f_dscp[0x1];
	u8 f_ecn[0x1];
	u8 reserved_at_106[0x1];
	u8 f_eth_prio[0x1];
	u8 ecn[0x2];
	u8 dscp[0x6];
	u8 udp_sport[0x10];

	u8 dei_cfi[0x1];
	u8 eth_prio[0x3];
	u8 sl[0x4];
	u8 vhca_port_num[0x8];
	u8 rmac_47_32[0x10];

	u8 rmac_31_0[0x20];
};

struct mlx5_ifc_qpc_bits {
	u8 state[0x4];
	u8 lag_tx_port_affinity[0x4];
	u8 st[0x8];
	u8 reserved_at_10[0x3];
	u8 pm_state[0x2];
	u8 reserved_at_15[0x1];
	u8 req_e2e_credit_mode[0x2];
	u8 offload_type[0x4];
	u8 end_padding_mode[0x2];
	u8 reserved_at_1e[0x2];

	u8 wq_signature[0x1];
	u8 block_lb_mc[0x1];
	u8 atomic_like_write_en[0x1];
	u8 latency_sensitive[0x1];
	u8 reserved_at_24[0x1];
	u8 drain_sigerr[0x1];
	u8 reserved_at_26[0x2];
	u8 pd[0x18];

	u8 mtu[0x3];
	u8 log_msg_max[0x5];
	u8 reserved_at_48[0x1];
	u8 log_rq_size[0x4];
	u8 log_rq_stride[0x3];
	u8 no_sq[0x1];
	u8 log_sq_size[0x4];
	u8 reserved_at_55[0x3];
	u8 ts_format[0x2];
	u8 reserved_at_5a[0x1];
	u8 rlky[0x1];
	u8 ulp_stateless_offload_mode[0x4];

	u8 counter_set_id[0x8];
	u8 uar_page[0x18];

	u8 reserved_at_80[0x3];
	u8 full_handshake[0x1];
	u8 cnak_reverse_sl[0x4];
	u8 user_index[0x18];

	u8 reserved_at_a0[0x3];
	u8 log_page_size[0x5];
	u8 remote_qpn[0x18];

	struct mlx5_ifc_ads_bits primary_address_path;

	struct mlx5_ifc_ads_bits secondary_address_path;

	u8 log_ack_req_freq[0x4];
	u8 reserved_at_384[0x4];
	u8 log_sra_max[0x3];
	u8 reserved_at_38b[0x2];
	u8 retry_count[0x3];
	u8 rnr_retry[0x3];
	u8 reserved_at_393[0x1];
	u8 fre[0x1];
	u8 cur_rnr_retry[0x3];
	u8 cur_retry_count[0x3];
	u8 reserved_at_39b[0x5];

	u8 reserved_at_3a0[0x20];

	u8 reserved_at_3c0[0x8];
	u8 next_send_psn[0x18];

	u8 reserved_at_3e0[0x4];
	u8 log_num_dci_stream_channels[0x4];
	u8 cqn_snd[0x18];

	u8 reserved_at_400[0x8];
	u8 deth_sqpn_or_initiator_src_dct[0x18];

	u8 reserved_at_420[0x20];

	u8 reserved_at_440[0x8];
	u8 last_acked_psn[0x18];

	u8 reserved_at_460[0x8];
	u8 ssn[0x18];

	u8 reserved_at_480[0x8];
	u8 log_rra_max[0x3];
	u8 reserved_at_48b[0x1];
	u8 atomic_mode[0x4];
	u8 rre[0x1];
	u8 rwe[0x1];
	u8 rae[0x1];
	u8 reserved_at_493[0x1];
	u8 page_offset[0x6];
	u8 reserved_at_49a[0x3];
	u8 cd_slave_receive[0x1];
	u8 cd_slave_send[0x1];
	u8 cd_master[0x1];

	u8 reserved_at_4a0[0x3];
	u8 min_rnr_nak[0x5];
	u8 next_rcv_psn[0x18];

	u8 reserved_at_4c0[0x8];
	u8 xrcd[0x18];

	u8 reserved_at_4e0[0x8];
	u8 cqn_rcv[0x18];

	u8 dbr_addr[0x40];

	u8 q_key[0x20];

	u8 reserved_at_560[0x5];
	u8 rq_type[0x3];
	u8 srqn_rmpn_xrqn[0x18];

	u8 reserved_at_580[0x8];
	u8 rmsn[0x18];

	u8 hw_sq_wqebb_counter[0x10];
	u8 sw_sq_wqebb_counter[0x10];

	u8 hw_rq_counter[0x20];

	u8 sw_rq_counter[0x20];

	u8 reserved_at_600[0x20];

	u8 reserved_at_620[0xf];
	u8 cgs[0x1];
	u8 cs_req[0x8];
	u8 cs_res[0x8];

	u8 dc_access_key[0x40];

	u8 reserved_at_640[0x3];
	u8 dbr_umem_valid[0x1];
	u8 reserved_at_660[0x1c];

	u8 reserved_at_680[0x80];

	u8 dbr_umem_id[0x20];
};

struct mlx5_ifc_create_qp_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];

	u8 syndrome[0x20];

	u8 reserved_at_40[0x8];
	u8 qpn[0x18];

	u8 ece[0x20];
};

struct mlx5_ifc_qpc_extension_bits {
	uint8_t reserved_at_0[0x2];
	uint8_t mmo[0x1];
	uint8_t reserved_at_3[0x5fd];
};

struct mlx5_ifc_qpc_pas_list_bits {
	uint8_t pas[0][0x40];
};

struct mlx5_ifc_qpc_extension_and_pas_list_bits {
	struct mlx5_ifc_qpc_extension_bits qpc_data_extension;
	uint8_t pas[0][0x40];
};

struct mlx5_ifc_create_qp_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];

	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];

	uint8_t qpc_ext[0x1];
	u8 reserved_at_41[0x7];
	u8 input_qpn[0x18];

	u8 reserved_at_60[0x20];
	u8 opt_param_mask[0x20];

	u8 ece[0x20];

	struct mlx5_ifc_qpc_bits qpc;

	u8 wq_umem_offset[0x40];

	u8 wq_umem_id[0x20];

	u8 wq_umem_valid[0x1];
	u8 reserved_at_861[0x1f];

	union {
		struct mlx5_ifc_qpc_pas_list_bits qpc_pas_list;
		struct mlx5_ifc_qpc_extension_and_pas_list_bits qpc_extension_and_pas_list;
	};
};

struct mlx5_ifc_dctc_bits {
	u8 reserved_at_0[0x4];
	u8 state[0x4];
	u8 reserved_at_8[0x18];

	u8 reserved_at_20[0x8];
	u8 user_index[0x18];

	u8 reserved_at_40[0x8];
	u8 cqn[0x18];

	u8 counter_set_id[0x8];
	u8 atomic_mode[0x4];
	u8 rre[0x1];
	u8 rwe[0x1];
	u8 rae[0x1];
	u8 atomic_like_write_en[0x1];
	u8 latency_sensitive[0x1];
	u8 rlky[0x1];
	u8 free_ar[0x1];
	u8 reserved_at_73[0xd];

	u8 reserved_at_80[0x8];
	u8 cs_res[0x8];
	u8 reserved_at_90[0x3];
	u8 min_rnr_nak[0x5];
	u8 reserved_at_98[0x8];

	u8 reserved_at_a0[0x8];
	u8 srqn_xrqn[0x18];

	u8 reserved_at_c0[0x8];
	u8 pd[0x18];

	u8 tclass[0x8];
	u8 reserved_at_e8[0x4];
	u8 flow_label[0x14];

	u8 dc_access_key[0x40];

	u8 reserved_at_140[0x5];
	u8 mtu[0x3];
	u8 port[0x8];
	u8 pkey_index[0x10];

	u8 reserved_at_160[0x8];
	u8 my_addr_index[0x8];
	u8 reserved_at_170[0x8];
	u8 hop_limit[0x8];

	u8 dc_access_key_violation_count[0x20];

	u8 reserved_at_1a0[0x14];
	u8 dei_cfi[0x1];
	u8 eth_prio[0x3];
	u8 ecn[0x2];
	u8 dscp[0x6];

	u8 reserved_at_1c0[0x20];

	u8 reserved_at_1e0[0x8];
	u8 max_outstanding_cennections[0x18];
};

struct mlx5_ifc_create_dct_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];

	u8 syndrome[0x20];

	u8 reserved_at_40[0x8];
	u8 dctn[0x18];

	u8 ece[0x20];
};

struct mlx5_ifc_create_dct_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];

	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];

	u8 reserved_at_40[0x40];

	struct mlx5_ifc_dctc_bits dct_context_entry;

	u8 reserved_at_280[0x300]; /** 0x300 is required to work with Winof2 devx */
};

struct mlx5_ifc_query_issi_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];

	u8 syndrome[0x20];

	u8 reserved_at_40[0x10];
	u8 current_issi[0x10];

	u8 reserved_at_60[0xa0];

	u8 reserved_at_100[76][0x8];
	u8 supported_issi_dw0[0x20];
};

struct mlx5_ifc_query_issi_in_bits {
	u8 opcode[0x10];
	u8 reserved_at_10[0x10];

	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];

	u8 reserved_at_40[0x40];
};

struct mlx5_ifc_rst2init_qp_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];

	u8 syndrome[0x20];

	u8 reserved_at_40[0x20];
	u8 ece[0x20];
};

struct mlx5_ifc_rst2init_qp_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];

	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];

	u8 reserved_at_40[0x8];
	u8 qpn[0x18];

	u8 reserved_at_60[0x20];

	u8 opt_param_mask[0x20];

	u8 ece[0x20];

	struct mlx5_ifc_qpc_bits qpc;

	u8 reserved_at_800[0x80];
};

struct mlx5_ifc_init2rtr_qp_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];

	u8 syndrome[0x20];

	u8 reserved_at_40[0x20];
	u8 ece[0x20];
};

struct mlx5_ifc_init2rtr_qp_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];

	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];

	u8 reserved_at_40[0x8];
	u8 qpn[0x18];

	u8 reserved_at_60[0x20];

	u8 opt_param_mask[0x20];

	u8 ece[0x20];

	struct mlx5_ifc_qpc_bits qpc;

	u8 reserved_at_800[0x80];
};

struct mlx5_ifc_rtr2rts_qp_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];

	u8 syndrome[0x20];

	u8 reserved_at_40[0x20];
	u8 ece[0x20];
};

struct mlx5_ifc_rtr2rts_qp_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];

	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];

	u8 reserved_at_40[0x8];
	u8 qpn[0x18];

	u8 reserved_at_60[0x20];

	u8 opt_param_mask[0x20];

	u8 ece[0x20];

	struct mlx5_ifc_qpc_bits qpc;

	u8 reserved_at_800[0x80];
};

struct mlx5_ifc_qp_2rst_out_bits {
	u8 status[0x8];
	u8 reserved_at_8[0x18];

	u8 syndrome[0x20];

	u8 reserved_at_40[0x40];
};

struct mlx5_ifc_qp_2rst_in_bits {
	u8 opcode[0x10];
	u8 uid[0x10];

	u8 reserved_at_20[0x10];
	u8 op_mod[0x10];

	u8 reserved_at_40[0x8];
	u8 qpn[0x18];

	u8 reserved_at_60[0x20];
};

struct mlx5_ifc_ud_adrs_vector_bits {
	u8 dc_key_up[0x20];

	u8 dc_key_low[0x20];

	u8 ext[0x1];
	u8 reserved_at_41[0x7];
	u8 destination_qp_dct[0x18];

	u8 static_rate[0x4];
	u8 sl_eth_prio[0x4];
	u8 fl[0x1];
	u8 mlid[0x7];
	u8 rlid_udp_sport[0x10];

	u8 reserved_at_80[0x20];

	u8 rmac_47_16[0x20];

	u8 rmac_15_0[0x10];
	u8 tclass[0x8];
	u8 hop_limit[0x8];

	u8 reserved_at_e0[0x1];
	u8 grh[0x1];
	u8 reserved_at_e2[0x2];
	u8 src_addr_index[0x8];
	u8 flow_label[0x14];

	u8 rgid_rip[16][0x8];
};

/* PRM section 8.28.53 CHANNEL_SERVICE Object */

#define MLX5_CHANNEL_SERVICE_NAME_LENGTH 128

enum {
	MLX5_CHANNEL_TRANSPORT_TYPE_UD = 0x0,
	MLX5_CHANNEL_TRANSPORT_TYPE_DC = 0x1,
};

struct mlx5_ifc_channel_service_obj_bits {
	u8 modify_field_select[0x40];

	u8 reserved_at_40[0x20];

	u8 allowed_vhca_id[0x10];
	u8 reserved_at_70[0x5];
	u8 transport_type[0x3];
	u8 reserved_at_78[0x7];
	u8 enforce_token[0x1];

	u8 service_qpn_or_dct[0x20];

	u8 service_qkey[0x20];

	u8 token[0x40];

	u8 dct_key[0x40];

	u8 reserved_at_140[0x1d];
	u8 channel_status[0x3];

	u8 max_num_connected_client[0x10];
	u8 num_connected_client[0x10];

	u8 reserved_at_180[0x80];

	u8 service_name[0x400];

	u8 reserved_at_600[0x200];
};

struct mlx5_ifc_create_channel_service_obj_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_channel_service_obj_bits ch_svc_obj;
};

struct mlx5_ifc_create_channel_service_obj_out_bits {
	struct mlx5_ifc_general_obj_out_cmd_hdr_bits hdr;
};

/* PRM section 8.28.54 CHANNEL_CONNECTION Object */

enum {
	MLX5_CHANNEL_CONNECT_BY_SERVICE_NAME = 0x0,
	MLX5_CHANNEL_CONNECT_BY_VHCA_ID = 0x1,
};

struct mlx5_ifc_channel_connection_obj_bits {
	u8 modify_field_select[0x40];

	u8 reserved_at_40[0x20];

	u8 dst_vhca_id[0x10];
	u8 reserved_at_70[0x5];
	u8 transport_type[0x3];
	u8 reserved_at_78[0x5];
	u8 connection_type[0x3];

	u8 my_qpn_or_dci[0x20];

	u8 reserved_at_a0[0x20];

	u8 match_token[0x40];

	u8 reserved_at_100[0x100];

	u8 service_name[0x400];

	u8 reserved_at_600[0x200];
};

struct mlx5_ifc_create_channel_connection_obj_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits hdr;
	struct mlx5_ifc_channel_connection_obj_bits ch_conn_obj;
};

struct mlx5_ifc_create_channel_connection_obj_out_bits {
	struct mlx5_ifc_general_obj_out_cmd_hdr_bits hdr;
};

struct mlx5_ifc_cqe_bits {
	u8 reserved_at_0[0x80];

	u8 smac_high[0x20];

	u8 checksum_ib_stride_index[0x10];
	u8 smac_low[0x10];

	u8 reserved_at_c0[0x1];
	u8 fl[0x1];
	u8 l3_hdr[0x2];
	u8 sl_roce_pktype[0x4];
	u8 rqpn[0x18];

	u8 reserved_at_e0[0x20];

	u8 lro_num_seg[0x8];
	u8 srqn_or_user_index[0x18];

	u8 immediate_invalidate_key_pkey_flow_table_meta[0x20];

	u8 application[0x8];
	u8 application_opcode[0x8];
	u8 application_info[0x10];

	u8 byte_cnt[0x20];

	u8 reserved_at_180[0x40];

	u8 send_wqe_opcode[0x8];
	u8 qpn[0x18];

	u8 wqe_counter[0x10];
	u8 signature_or_validity_iteration_count[0x8];
	u8 opcode[0x4];
	u8 cqe_format[0x1];
	u8 se[0x1];
	u8 owner[0x1];
};

struct mlx5_ifc_cqe_error_layout_bits {
	u8 reserved_at_0[0x1a0];

	u8 cqe_error_syndrom[0x20];

	u8 send_wqe_opcode[0x8];
	u8 qpn[0x18];

	u8 wqe_counter[0x10];
	u8 signature[0x8];
	u8 opcode[0x4];
	u8 reserved_at_1ec[0x2];
	u8 se[0x1];
	u8 owner[0x1];
};

struct mlx5_ifc_cqe_error_syndrom_bits {
	u8 hw_error_syndrom[0x8];
	u8 hw_syndrom_type[0x4];
	u8 reserved_at_c[0x4];
	u8 vendor_error_syndrom[0x8];
	u8 syndrom[0x8];
};

/* PRM section 29.30 ESW Functions */

struct mlx5_ifc_host_params_context_bits {
	u8         host_number[0x8];
	u8         reserved_at_8[0x4];
	u8         host_vf_info_valid[0x1];
	u8         host_pf_not_exist[0x1];
	u8         host_pf_vhca_id_valid[0x1];
	u8         host_pf_disabled[0x1];
	u8         host_num_vfs[0x10];

	u8         host_total_vfs[0x10];
	u8         host_pci_bus[0x10];

	u8         host_pf_vhca_id[0x10];
	u8         host_pci_device[0x10];

	u8         reserved_at_60[0x10];

	u8         host_pci_function[0x10];

	u8         host_pci_vf_stride[0x10];
	u8         host_pci_first_vf_offset[0x10];

	u8         reserved_at_A0[0x160];
};

struct mlx5_ifc_query_esw_functions_in_bits {
	u8         opcode[0x10];
	u8         uid[0x10];

	u8         reserved_at_20[0x10];
	u8         op_mod[0x10];

	u8         reserved_at_40[0x40];
};

struct mlx5_ifc_query_esw_functions_out_bits {
	u8         status[0x8];
	u8         reserved_at_8[0x18];

	u8         syndrome[0x20];

	u8         reserved_at_40[0x40];

	struct mlx5_ifc_host_params_context_bits host_params;

	u8         reserved_at_280[0x180];
	u8         host_sf_enable[][0x40];
};

/* PRM 0_54_DraftB ipsec */

struct mlx5_ifc_ipsec_cap_bits {
	u8         ipsec_full_offload[0x1];
	u8         ipsec_crypto_offload[0x1];
	u8         ipsec_esn[0x1];
	u8         ipsec_crypto_esp_aes_gcm_256_encrypt[0x1];
	u8         ipsec_crypto_esp_aes_gcm_128_encrypt[0x1];
	u8         ipsec_crypto_esp_aes_gcm_256_decrypt[0x1];
	u8         ipsec_crypto_esp_aes_gcm_128_decrypt[0x1];
	u8         reserved_at_7[0x4];
	u8         log_max_ipsec_offload[0x5];
	u8         reserved_at_10[0x10];

	u8         min_log_ipsec_full_replay_window[0x8];
	u8         max_log_ipsec_full_replay_window[0x8];
	u8         reserved_at_30[0x7d0];
};


enum {
	MLX5_IPSEC_OBJECT_ICV_LEN_16B = 0x0,
	MLX5_IPSEC_OBJECT_ICV_LEN_12B = 0x1,
	MLX5_IPSEC_OBJECT_ICV_LEN_8B = 0x2,
};

struct mlx5_ifc_ipsec_obj_bits {
	u8         modify_field_select[0x40];

	u8         full_offload[0x1];
	u8         reserved_at_41[0x1];
	u8         esn_en[0x1];
	u8         esn_overlap[0x1];
	u8         reserved_at_44[0x2];
	u8         icv_length[0x2];
	u8         reserved_at_48[0x4];
	u8         aso_return_reg[0x4];
	u8         reserved_at_50[0x10];

	u8         esn_msb[0x20];

	u8         reserved_at_80[0x8];
	u8         dekn[0x18];

	u8         salt[0x20];

	u8         implicit_iv[0x40];

	u8         reserved_at_160[0x8];
	u8         ipsec_aso_access_pd[0x18];

	u8         reserved_at_200[0x40];

	u8         ipsec_aso[0x200];

	u8         reserved_at_300[0x4A0];
};

struct mlx5_ifc_ipsec_aso_obj_bits {
	u8         valid[0x1];
	u8         reserved_at_1[0x1];
	u8         mode[0x2];
	u8         window_size[0x2];
	u8         soft_lifetime_arm[0x1];
	u8         hard_lifetime_arm[0x1];
	u8         remove_flow_enable[0x1];
	u8         ens_event_arm[0x1];
	u8         reserved_at_16[0x16];

	u8         remove_flow_packet_count[0x20];

	u8         remove_flow_soft_lifetime[0x20];

	u8         reserved_at_96[0x40];

	u8         mode_parameter[0x20];

	u8         replay_protection_window[0x100];
};

struct mlx5_ifc_create_ipsec_obj_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits general_obj_in_cmd_hdr;
	struct mlx5_ifc_ipsec_obj_bits ipsec_object;
};

struct mlx5_ifc_query_ipsec_obj_out_bits {
	struct mlx5_ifc_general_obj_out_cmd_hdr_bits general_obj_out_cmd_hdr;
	struct mlx5_ifc_ipsec_obj_bits ipsec_object;
};

struct mlx5_ifc_modify_ipsec_obj_in_bits {
	struct mlx5_ifc_general_obj_in_cmd_hdr_bits general_obj_in_cmd_hdr;
	struct mlx5_ifc_ipsec_obj_bits ipsec_object;
};

#ifdef __cplusplus
}
#endif

#endif /* MLX5_PRM_H_ */
