/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef MLX5_PRM_DATA_PATH_H_
#define MLX5_PRM_DATA_PATH_H_

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef u8
#define u8 uint8_t
#endif

#ifndef __linux__
enum {
	MLX5_SEND_WQE_BB = 64,
	MLX5_SEND_WQE_SHIFT = 6,
};

struct mlx5_wqe_ctrl_seg {
	uint32_t opmod_idx_opcode;
	uint32_t qpn_ds;
	uint8_t signature;
	uint8_t rsvd[2];
	uint8_t fm_ce_se;
	uint32_t imm;
};

struct mlx5_wqe_umr_ctrl_seg {
	uint8_t flags;
	uint8_t rsvd0[3];
	__be16 klm_octowords;
	union {
		__be16 translation_offset;
		__be16 bsf_octowords;
	};
	__be64 mkey_mask;
	uint8_t rsvd1[32];
};

struct mlx5_wqe_umr_klm_seg {
	/* up to 2GB */
	__be32 byte_count;
	__be32 mkey;
	__be64 address;
};

union mlx5_wqe_umr_inline_seg {
	struct mlx5_wqe_umr_klm_seg klm;
};

struct mlx5_wqe_mkey_context_seg {
	uint8_t free;
	uint8_t reserved1;
	uint8_t access_flags;
	uint8_t sf;
	__be32 qpn_mkey;
	__be32 reserved2;
	__be32 flags_pd;
	__be64 start_addr;
	__be64 len;
	__be32 bsf_octword_size;
	__be32 reserved3[4];
	__be32 translations_octword_size;
	uint8_t reserved4[3];
	uint8_t log_page_size;
	__be32 reserved;
	union mlx5_wqe_umr_inline_seg inseg[0];
};

struct mlx5_wqe_data_seg {
	__be32 byte_count;
	__be32 lkey;
	__be64 addr;
};

enum {
	MLX5_WQE_CTRL_CQ_UPDATE = 2 << 2,
	MLX5_WQE_CTRL_SOLICITED = 1 << 1,
	MLX5_WQE_CTRL_FENCE = 4 << 5,
};

enum {
	MLX5_RCV_DBR = 0,
	MLX5_SND_DBR = 1,
};

enum mlx5_opcode_type {
	MLX5_OPCODE_NOP = 0x00,
	MLX5_OPCODE_SEND_INVAL = 0x01,
	MLX5_OPCODE_RDMA_WRITE = 0x08,
	MLX5_OPCODE_RDMA_WRITE_IMM = 0x09,
	MLX5_OPCODE_SEND = 0x0a,
	MLX5_OPCODE_SEND_IMM = 0x0b,
	MLX5_OPCODE_LSO = 0x0e,
	MLX5_OPCODE_RDMA_READ = 0x10,
	MLX5_OPCODE_ATOMIC_CS = 0x11,
	MLX5_OPCODE_ATOMIC_FA = 0x12,
	MLX5_OPCODE_ATOMIC_MASK_CS = 0x14,
	MLX5_OPCODE_ATOMIC_MASK_FA = 0x15,
	MLX5_OPCODE_BIND_MW = 0x18,
	MLX5_OPCODE_FMR = 0x19,
	MLX5_OPCODE_LOCAL_INVAL = 0x1b,
	MLX5_OPCODE_CONFIG_CMD = 0x1f,
	MLX5_OPCODE_UMR = 0x25,

	MLX5_RECV_OPCODE_RDMA_WRITE_IMM = 0x00,
	MLX5_RECV_OPCODE_SEND = 0x01,
	MLX5_RECV_OPCODE_SEND_IMM = 0x02,
	MLX5_RECV_OPCODE_SEND_INVAL = 0x03,

	MLX5_CQE_OPCODE_ERROR = 0x1e,
	MLX5_CQE_OPCODE_RESIZE = 0x16,

	MLX5_OPCODE_INVALID = 0xff
};

enum {
	MLX5_WQE_UMR_CTRL_MKEY_MASK_LEN			= 1 << 0,
	MLX5_WQE_UMR_CTRL_MKEY_MASK_START_ADDR		= 1 << 6,
	MLX5_WQE_UMR_CTRL_MKEY_MASK_SIG_ERR		= 1 << 9,
	MLX5_WQE_UMR_CTRL_MKEY_MASK_BSF_ENABLE		= 1 << 12,
	MLX5_WQE_UMR_CTRL_MKEY_MASK_MKEY		= 1 << 13,
	MLX5_WQE_UMR_CTRL_MKEY_MASK_QPN			= 1 << 14,
	MLX5_WQE_UMR_CTRL_MKEY_MASK_ACCESS_LOCAL_WRITE	= 1 << 18,
	MLX5_WQE_UMR_CTRL_MKEY_MASK_ACCESS_REMOTE_READ	= 1 << 19,
	MLX5_WQE_UMR_CTRL_MKEY_MASK_ACCESS_REMOTE_WRITE	= 1 << 20,
	MLX5_WQE_UMR_CTRL_MKEY_MASK_ACCESS_ATOMIC	= 1 << 21,
	MLX5_WQE_UMR_CTRL_MKEY_MASK_FREE		= 1 << 29,
};

enum {
	MLX5_WQE_UMR_CTRL_FLAG_INLINE =			1 << 7,
	MLX5_WQE_UMR_CTRL_FLAG_CHECK_FREE =		1 << 5,
	MLX5_WQE_UMR_CTRL_FLAG_TRNSLATION_OFFSET =	1 << 4,
	MLX5_WQE_UMR_CTRL_FLAG_CHECK_QPN =		1 << 3,
};

#endif /* __linux__ */

// MSVC can't compile member structs if they have [0] at the end and they are not the last member
// See mlx5_wqe_mkey_context_seg above and it's usage in the code.
struct mlx5_wqe_mkey_context_seg_member {
	uint8_t free;
	uint8_t reserved1;
	uint8_t access_flags;
	uint8_t sf;
	__be32 qpn_mkey;
	__be32 reserved2;
	__be32 flags_pd;
	__be64 start_addr;
	__be64 len;
	__be32 bsf_octword_size;
	__be32 reserved3[4];
	__be32 translations_octword_size;
	uint8_t reserved4[3];
	uint8_t log_page_size;
	__be32 reserved;
};

#ifndef __linux__ /* For Linux they are defined in mlx5dv.h */

enum {
	MLX5_CQE_OWNER_MASK	= 1,
	MLX5_CQE_REQ		= 0,
	MLX5_CQE_RESP_WR_IMM	= 1,
	MLX5_CQE_RESP_SEND	= 2,
	MLX5_CQE_RESP_SEND_IMM	= 3,
	MLX5_CQE_RESP_SEND_INV	= 4,
	MLX5_CQE_RESIZE_CQ	= 5,
	MLX5_CQE_NO_PACKET	= 6,
	MLX5_CQE_REQ_ERR	= 13,
	MLX5_CQE_RESP_ERR	= 14,
	MLX5_CQE_INVALID	= 15,
};

enum {
	MLX5_CQ_DB_REQ_NOT_SOL	= 1 << 24,
	MLX5_CQ_DB_REQ_NOT	= 0 << 24,
};

enum {
	MLX5_CQ_DOORBELL	= 0x20
};

#endif /* Windows */

enum {
	MLX5_CQ_SET_CI	= 0,
	MLX5_CQ_ARM_DB	= 1,
};

#define MLX5_CQE_OPCODE_SHIFT 4

struct mlx5_64b_cqe {
	struct {
		uint8_t		reserved_at_0[2];
		__be16		wqe_id;

		uint8_t		reserved_at_4[12];

		uint8_t		reserved_at_16;
		uint8_t		ml_path;
		uint8_t		rsvd20[2];

		uint8_t		rsvd22[2];
		__be16		slid;

		__be32		flags_rqpn;

		uint8_t		hds_ip_ext;
		uint8_t		l4_hdr_type_etc;
		__be16		vlan_info;
	};
	__be32		srqn_uidx;

	__be32		imm_inval_pkey;

	uint8_t		app;
	uint8_t		app_op;
	__be16		app_info;

	__be32		byte_cnt;

	__be64		timestamp;

	__be32		sop_drop_qpn;

	__be16		wqe_counter;
	uint8_t		signature;
	uint8_t		op_own;
};

struct mlx5_error_cqe {
	uint8_t rsvd0[54];
	uint8_t vendor_err_synd;
	uint8_t syndrome;
	uint8_t rsvd56[8];
};

#ifdef __cplusplus
}
#endif

#endif /* MLX5_PRM_DATA_PATH_H_ */
