/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#pragma once

#include <stddef.h>

#include <priv_doca_compat.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief This method allocates an aligned buffer
 * @param [in] alignment
 * @param [in] size in bytes
 * @return buffer or null in case of no memory
 */
DOCA_PRIVATE
void *priv_doca_allocate_aligned_buffer(size_t alignment, size_t size);

/**
 * @brief This method releases an aligned buffer
 * @param [in] buffer to release (can be null)
 */
DOCA_PRIVATE
void priv_doca_free_aligned_buffer(void *buffer);

/**
 * @brief This method implements memory barrier.
 */
DOCA_PRIVATE
void priv_doca_memory_barrier();

#ifdef DOCA_ARCH_ARM
#define priv_doca_udma_to_device_barrier() asm volatile("dsb st" ::: "memory")
#else /* X86_64 */
#define priv_doca_udma_to_device_barrier() asm volatile("" ::: "memory")
#endif

#ifdef DOCA_ARCH_ARM
#define priv_doca_mmio_flush_writes() asm volatile("dsb st" ::: "memory")
#else /* X86_64 */
#define priv_doca_mmio_flush_writes() asm volatile("sfence" ::: "memory")
#endif

#define priv_doca_mmio_wc_start priv_doca_mmio_flush_writes

#ifdef __cplusplus
}
#endif
