/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#pragma once

#include <atomic>

namespace os_abstraction
{

class doca_spin_lock
{
public:
	/**
	 * Constructor
	 */
	doca_spin_lock() noexcept = default;

	/**
	 * Destructor
	 */
	~doca_spin_lock() noexcept = default;

	/**
	 * @brief This method spins until the lock is locked
	 */
	inline void lock() noexcept
	{
		while (m_lock.test_and_set()) {
			/* spin */
		}
	}

	/**
	 * @brief This method unlocks the spin lock
	 */
	inline void unlock() noexcept
	{
		m_lock.clear();
	}

private:
	std::atomic_flag m_lock = ATOMIC_FLAG_INIT;
};

} // namespace os_abstraction