/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */
#include <doca_build_config.h>

#include <stdexcept>
#include <malloc.h>
#include <stdlib.h>
#include <unistd.h>
#ifdef DOCA_ARCH_X86
#include <immintrin.h>
#endif

#include <priv_doca_os_abstraction_memory.hpp>

void *
priv_doca_allocate_aligned_buffer(size_t alignment, size_t size)
{
	if (alignment == 0 || size == 0)
		return nullptr;

	return memalign(alignment, size);
}

void
priv_doca_free_aligned_buffer(void *buffer)
{
	if (buffer != nullptr)
		free(buffer);
}

void
priv_doca_memory_barrier()
{
#ifdef DOCA_ARCH_ARM
	asm volatile("" : : : "memory");
#else /* !DOCA_ARCH_ARM */
	_mm_sfence();
#endif /* DOCA_ARCH_ARM */
}
