/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DOCA_FLOW_LAYER_H_
#define DOCA_FLOW_LAYER_H_

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

struct doca_flow_port;
struct doca_flow_pipe;

/**
 * @brief Initialize doca flow layer.
 *
 * @return
 * 0 on success, negative on failure.
 */
int
doca_flow_layer_init();

/**
 * @brief Destroy doca flow layer.
 */
void
doca_flow_layer_destroy(void);

/**
 * @brief gets the port of a pipe.
 *
 * @param pipe
 * Pointer to a pipe.
 * @return
 * Pointer to a port if valid, or null.
 */
struct doca_flow_port *
doca_flow_layer_get_pipe_port(struct doca_flow_pipe *pipe);

/**
 * @brief Get the default DPDK field chain provider.
 *
 * @return
 * Pointer to the field chain provider.
 */
struct engine_fcp *
doca_flow_layer_get_default_match(void);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* DOCA_FLOW_LAYER_H_ */
