/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include <errno.h>
#include <string.h>

#include "engine_field_mapping.h"

#include "doca_log.h"

DOCA_LOG_REGISTER(dpdk_field_mapping);

static uint32_t dpdk_field_offset_map[DF_SUP_FOCUS_MAX][DF_SUP_PROTO_MAX][DF_SUP_PROTO_FIELD_MAX];

int
dpdk_field_mapping_init(void)
{
	memset(dpdk_field_offset_map, 0, sizeof(dpdk_field_offset_map));
	return 0;
}

void
dpdk_field_mapping_destroy(void)
{

}

static bool
is_opcode_supported(const struct engine_field_opcode *opcode)
{
	if (opcode->pkt.group != ENGINE_FIELD_OPCODE_GROUP_PACKET)
		return false;
	if (opcode->pkt.type >= DF_SUP_USER_DEFINED_TYPE_MAX)
		return false;
	if (opcode->pkt.focus >= DF_SUP_FOCUS_MAX)
		return false;
	if (opcode->pkt.proto >= DF_SUP_PROTO_MAX)
		return false;
	if (opcode->pkt.field >= DF_SUP_PROTO_FIELD_MAX)
		return false;
	return true;
}

int
dpdk_field_mapping_set_offset(const struct engine_field_opcode *opcode, uint32_t offset)
{
	if (!is_opcode_supported(opcode))
		return -ENOTSUP;

	dpdk_field_offset_map[opcode->pkt.focus][opcode->pkt.proto][opcode->pkt.field] = offset;
	DOCA_DLOG_DBG("Dpdk field mapping set opcode=0x%lx offset=%u)", opcode->opcode, offset);
	return 0;
}

int
dpdk_field_mapping_get_offset(const struct engine_field_opcode *opcode, uint32_t *offset)
{
	if (!is_opcode_supported(opcode))
		return -ENOTSUP;

	*offset = dpdk_field_offset_map[opcode->pkt.focus][opcode->pkt.proto][opcode->pkt.field];
	return 0;
}
