/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPDK_FIELD_MAPPING_H_
#define DPDK_FIELD_MAPPING_H_

#include <stdbool.h>
#include <stdint.h>

#include "engine_field_opcode.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief Initialized field mapping
 *
 * @return
 * 0 on success, negative on failure.
 */
int
dpdk_field_mapping_init(void);

/**
 * @brief Destroy field mapping.
 *
 */
void
dpdk_field_mapping_destroy(void);

/**
 * @brief Sets the field offset
 *
 * @param opcode
 * Field opcode which is a unique identifier of a field.
 * @param offset
 * The offset of a field in a dpdk relevant struct.
 * @return
 * 0 on success, negative on failure.
 */
int
dpdk_field_mapping_set_offset(const struct engine_field_opcode *opcode, uint32_t offset);

/**
 * @brief Gets the field offset
 *
 * @param opcode
 * Field opcode which is a unique identifier of a field.
 * @param offset
 * Pointer to the output of field mapped offset value.
 * @return
 * 0 on success, negative on failure.
 */
int
dpdk_field_mapping_get_offset(const struct engine_field_opcode *opcode, uint32_t *offset);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* DPDK_FIELD_MAPPING_H_ */
