/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPDK_FLOW_SINGLE_H_
#define DPDK_FLOW_SINGLE_H_

#include "dpdk_table.h"
#include "dpdk_flow.h"

#include <rte_spinlock.h>

#ifdef __cplusplus
extern "C" {
#endif

struct dpdk_flow_single {
	uint32_t group_id;
	struct dpdk_flow_queue *queue;
	rte_spinlock_t *lock;
	struct dpdk_table dpdk_table;
	struct dpdk_flow_tracker tracker;
	bool in_hw;
	bool mark_for_delete;
};

int
dpdk_flow_single_create(struct dpdk_port *port,
			struct dpdk_flow_params *params,
			struct dpdk_flow_queue *queue,
			rte_spinlock_t *queue_lock,
			uint8_t nb_actions,
			uint32_t nb_flows,
			struct dpdk_flow_single *flow_single);

void
dpdk_flow_single_destroy(struct dpdk_port *port,
			 struct dpdk_flow_queue *queue,
			 rte_spinlock_t *queue_lock,
			 struct dpdk_flow_single *flow_single);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* DPDK_FLOW_SINGLE_H_ */
