/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPDK_FWD_GROUPS_H_
#define DPDK_FWD_GROUPS_H_

#ifdef __cplusplus
extern "C" {
#endif

struct dpdk_fwd_groups;
struct dpdk_port;

struct dpdk_fwd_groups_cfg {
	struct dpdk_port *port;
	struct dpdk_flow_queue *management_queue;
	void *management_lock;
};

enum dpdk_fwd_type {
	DPDK_FWD_JUMP,
	DPDK_FWD_QUEUE,
	DPDK_FWD_PORT,
	DPDK_FWD_RSS,
	DPDK_FWD_DROP,
};

struct dpdk_fwd_groups_key {
	uint16_t port_id;
	enum dpdk_fwd_type type;
	union {
		struct {
			uint32_t id;
		} port;
		struct {
			uint32_t index;
		} queue;
		struct {
			uint32_t group;
		} jump;
		struct {
			uint32_t shared_rss_id;
		} rss;
	};
};

struct dpdk_fwd_groups_req {
	struct dpdk_fwd_groups_key key;
	bool ingress:1;
	bool egress:1;
	bool transfer:1;
	uint32_t group;
	uint32_t priority;
};

/**
 * @brief Create fwd groups based on cfg
 *
 * @param cfg
 * Pointer to configuration
 * @return
 * Valid port pointer on success, otherwise NULL.
 */
struct dpdk_fwd_groups *
dpdk_fwd_groups_create(struct dpdk_fwd_groups_cfg *cfg);

/**
 * @brief Destroy fwd groups
 *
 * @param fwd_groups
 * Pointer to fwd_groups
 * @return
 *
 */
void
dpdk_fwd_groups_destroy(struct dpdk_fwd_groups *fwd_groups);

/**
 * @brief Get fwd groups
 *
 * @param fwd_groups
 * Pointer to fwd_groups
 * @param req
 * Request to request
 * @param group_id
 * Pointer to group_id
 * @return
 * 0 on success and negative value otherwise
 */
int
dpdk_fwd_groups_get(struct dpdk_fwd_groups *fwd_groups, struct dpdk_fwd_groups_req *req, uint32_t *group_id);

int
dpdk_fwd_groups_cfg_get_by_id(uint16_t port_id, void **mgmt_queue, void **mgmt_lock);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* DPDK_FWD_GROUPS_H_ */
