/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPDK_GROUP_POOL_H_
#define DPDK_GROUP_POOL_H_

#include <stdint.h>
#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

struct dpdk_group_pool_cfg {
	uint32_t min_group_id;
};

/**
 * @brief Create dpdk_group_pool
 *
 * @return
 * pointer of dpdk_group_pool, NULL on failure.
 */
struct dpdk_group_pool *
dpdk_group_pool_create(struct dpdk_group_pool_cfg *cfg);

/**
 * @brief Destroy dpdk_group_pool
 *
 * @return
 */
void
dpdk_group_pool_destroy(struct dpdk_group_pool *group_pool);

/**
 * @brief Allocate group_id from dpdk_group_pool
 *
 * @return
 * 0 on success, negative on failure.
 */
int
dpdk_group_pool_alloc(struct dpdk_group_pool *group_pool, uint32_t *group_id);

/**
 * @brief Free group_id to dpdk_group_pool
 *
 * @return
 * 0 on success, negative on failure.
 */
int
dpdk_group_pool_free(struct dpdk_group_pool *group_pool, uint32_t group_id);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* DPDK_GROUP_POOL_H_ */
