/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPDK_INTERNAL_META_H_
#define DPDK_INTERNAL_META_H_

#include "dpdk_resource_manager.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Max meta segments. */
#define DPDK_META_MAX_SEGMENTS 16

/**
 * @brief Get ops of internal meta
 *
 * @return
 * Pointer to ops, null on error.
 */
struct dpdk_resource_manager_ops *dpdk_internal_meta_get_ops();

/**
 * @brief Get tag index of user tag area
 *
 * @return
 * Tag index of the first user tag.
 */
uint32_t dpdk_internal_meta_user_tag_index(void);

/**
 * @brief Internal meta registration finished, registers allocated.
 *
 * Unable to register new once allocation done.
 *
 * @return allocation finished.
 */
bool dpdk_internal_meta_allocate_done(void);

/**
 * @brief Get intenrl meta info.
 *
 * @param[out] bit_width
 *  registered bit width.
 * @return
 *  Bit offset start from tag 0.
 */
int dpdk_internal_meta_get(uint32_t meta_id, uint32_t *bit_width);

/**
 * @brief Get tag index of registered internal meta
 *
 * @return tag index of registered internal meta.
 */
int dpdk_internal_meta_tag_index(uint32_t meta_id);

/**
 * @brief Build data or mask of registered internal meta
 *
 * @param[out] out
 *  Tag value.
 * @return
 *  0 on success, negative errno on error.
 */
int dpdk_internal_meta_build(uint32_t meta_id, uint32_t data, uint32_t *out);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* DPDK_INTERNAL_META_H_ */
