/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPDK_LAYER_H_
#define DPDK_LAYER_H_

#include <stdint.h>
#include "dpdk_field_mapping.h"

#include "dpdk_flow.h"
#include "dpdk_port.h"
#include "dpdk_resource_manager.h"
#include "dpdk_table.h"

#ifdef __cplusplus
extern "C" {
#endif

struct dpdk_layer_cfg {
	struct dpdk_flow_cfg flow_cfg;
	struct dpdk_table_cfg table_cfg;
};

/**
 * @brief Registers dpdk layer.
 *
 * Connect the dpdk layer to the engine.
 *
 * @param cfg
 * Pointer to layer configuration.
 * @return
 * 0 on success, negative on failure.
 */
int
dpdk_layer_register(struct dpdk_layer_cfg *cfg);

/**
 * @brief Unregisters dpdk layer.
 *
 * Disconnect the dpdk layer from the engine.
 */
void
dpdk_layer_unregister();


#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* DPDK_LAYER_H_ */
