/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPDK_RESOURCE_MANAGER_H_
#define DPDK_RESOURCE_MANAGER_H_

#include <stdint.h>

#include "engine_shared_resources.h"

#ifdef __cplusplus
extern "C" {
#endif

struct dpdk_resource_manager_ops {
	int (*resources_init)(uint32_t nr_resources);
	/**< shared resources initialization functionality */

	int (*resource_create)(uint32_t id, struct engine_shared_resource_cfg *res_cfg);
	/**< shared resource creation functionality */

	int (*resource_modify)(uint32_t id, struct engine_shared_resource_cfg *res_cfg);
	/**< shared resource modification functionality */

	int (*resource_query)(uint32_t id,
			      struct engine_shared_resource_query_result *res_query_rslt);
	/**< shared resource query functionality */

	int (*resource_destroy)(uint32_t id);
	/**< destroy shared resource functionality */

	int (*resources_cleanup)(void);
	/**< shared resources cleanup functionality */
};

/**
 * @brief Register the dpdk resource manager
 *
 * @return
 * 0 on success, negative on failure.
 */
int
dpdk_resource_manager_register();

/**
 * @brief Unregister the dpdk resource manager
 *
 * @return
 * 0 on success, negative on failure.
 */
void
dpdk_resource_manager_unregister();

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* DPDK_RESOURCE_MANAGER_H_ */
