/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPDK_SHARED_CRYPTO_H_
#define DPDK_SHARED_CRYPTO_H_

#include "dpdk_resource_manager.h"
#ifdef DPDK_SUPPORTS_NISP
#include <rte_pmd_mlx5_nisp.h>
#else
#include <rte_ethdev.h>
#include <rte_pmd_mlx5.h>

#define RTE_PMD_MLX5_FLOW_ITEM_TYPE_NISP	  (INT_MIN + 1000000 + 1)
#define RTE_PMD_MLX5_FLOW_ITEM_TYPE_AUDP	  (INT_MIN + 1000000 + 2)
#define RTE_PMD_MLX5_FLOW_ITEM_TYPE_NISP_SYNDROME (INT_MIN + 1000000 + 3)
#define RTE_RMD_MLX5_RTE_FLOW_ACTION_TYPE_NISP    (INT_MIN + 1000000 + 1)

/**< NISP action reformat operation selection. */
enum rte_pmd_mlx5_nisp_reformat_type  {
        RTE_PMD_MLX5_NISP_REFORMAT_TYPE_NONE = 0,
        RTE_PMD_MLX5_NISP_REFORMAT_TYPE_ENCAP,
        RTE_PMD_MLX5_NISP_REFORMAT_TYPE_DECAP,
};

/**< NISP action crypto operation selection. */
enum rte_pmd_mlx5_nisp_crypto_type {
        RTE_PMD_MLX5_NISP_CRYPTO_TYPE_NONE = 0,
        RTE_PMD_MLX5_NISP_CRYPTO_TYPE_ENCRYPT,
        RTE_PMD_MLX5_NISP_CRYPTO_TYPE_DECRYPT,
};

struct rte_pmd_mlx5_flow_action_nisp {
        enum rte_pmd_mlx5_nisp_reformat_type reformat_type;
        enum rte_pmd_mlx5_nisp_crypto_type crypto_type;
        size_t reformat_data_sz;
        void *reformat_data;
        size_t key_sz;
        void *key;
        uint32_t spi;
        struct rte_flow_action fwd;
};

struct mlx5_flow_item_nisp {
        rte_be32_t dw[10];
};

struct mlx5_flow_item_audp {
        rte_be32_t dw[6];
};

struct mlx5_flow_item_nisp_syndrome {
        union {
                uint8_t syndrome;
                uintptr_t pad;
        };
};

struct rte_flow_action_handle *
rte_pmd_mlx5_nisp_flow_action_create(uint16_t port_id,
				     const struct rte_flow_indir_action_conf *domain,
				     const struct rte_flow_action *action,
				     struct rte_flow_error *error);

int
rte_pmd_mlx5_nisp_flow_action_destroy(uint16_t port_id,
				      const struct rte_flow_action_handle *handle,
				      struct rte_flow_error *error);
#endif

/**
 * @brief Get ops of shared NISP action
 *
 * @return
 * Pointer to ops, null on error.
 */
struct dpdk_resource_manager_ops *dpdk_shared_crypto_get_ops(void);

struct rte_flow_action_handle *
dpdk_shared_crypto_translate(uint32_t id);

#endif /* DPDK_SHARED_CRYPTO_H_ */
