/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPDK_DBG_H_
#define DPDK_DBG_H_

enum {
	DEBUG_MBUFF = (1 << 0),
	DEBUG_RTE_FLOW = (1 << 1),
	DEBUG_MATCH = (1 << 2),
	DEBUG_ACTIONS = (1 << 3),
};

#define MAX_TMP_BUFF 1024

#define log_buff(fmt, args...)                        \
	sprintf(buff + strlen(buff), fmt, ##args)

#define log_ipv4(item, ipv4_addr)		\
	log_buff("%s%d.%d.%d.%d ", item,	\
		(ipv4_addr >> 24) & 0xFF,		\
		(ipv4_addr >> 16) & 0xFF,		\
		(ipv4_addr >> 8) & 0xFF,		\
		ipv4_addr & 0xFF)
#define log_mac(item, eth_addr)						\
	log_buff("%s%02X:%02X:%02X:%02X:%02X:%02X ",		\
		item, eth_addr[0], eth_addr[1], eth_addr[2],		\
		eth_addr[3], eth_addr[4], eth_addr[5])

struct dump_pkt_hdr {
	uint8_t protocol;
	uint8_t (*dump_hdr)(uint8_t *head, uint32_t *len, char *buffer);
};

enum {
	DUMP_ETH = 0,
	DUMP_IPV4,
	DUMP_UDP,
	DUMP_TCP,
	DUMP_VXLAN,
	DUMP_GRE,
	DUMP_END,
};

void
__dpdk_dbg_rte_flow_dump(const char *name, uint16_t port_id,
		   const struct rte_flow_attr *attr,
		   const struct rte_flow_item items[],
		   const struct rte_flow_action actions[],
		   FILE *f);
void
__dpdk_dbg_rte_mbuff_dump(const char *name, struct rte_mbuf *mb, FILE *f);

void
__dpdk_dbg_pipe_config_dump(const struct doca_flow_match *match,
		 const struct doca_flow_actions *actions,
		 const struct doca_flow_action_descs *desc,
		 const struct doca_flow_monitor *monitor,
		 const struct doca_flow_fwd *fwd);

#ifdef DOCA_LOGGING_ALLOW_DLOG

#define dpdk_dbg_rte_flow_dump(...) __dpdk_dbg_rte_flow_dump(__VA_ARGS__)
#define dpdk_dbg_rte_mbuff_dump(...) __dpdk_dbg_rte_mbuff_dump(__VA_ARGS__)
#define dpdk_dbg_pipe_config_dump(...) __dpdk_dbg_pipe_config_dump(__VA_ARGS__)

#else

#define dpdk_dbg_rte_flow_dump(...)
#define dpdk_dbg_rte_mbuff_dump(...)
#define dpdk_dbg_pipe_config_dump(...)

#endif

#endif /* DPDK_DBG_H_ */
