/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef _DPDK_FLOW_HWS_H_
#define _DPDK_FLOW_HWS_H_

#define MAX_QUEUE_DEPTH 1024
#define DEFAULT_QUEUE_DEPTH 128
#define PULL_TIMEOUT_US (10000)
#define MAX_HWS_ACTION_TEMPLATE   (4)

typedef void (*hws_process_queue_ctx_cb)(void *ctx);

struct flow_queue_ctx_item {
	LIST_ENTRY(flow_queue_ctx_item) next;
	bool in_use;
	void *ctx;
	void *usr_ctx;  /* user input during adding pipe_entrty */
	hws_process_queue_ctx_cb cb;
	enum doca_flow_entry_op op;
};

struct flow_queue_ctx_array {
	LIST_HEAD(, flow_queue_ctx_item) free_list;
	uint32_t array_len;
	struct flow_queue_ctx_item items[0];
};

struct dpdk_hws_item {
	LIST_ENTRY(dpdk_hws_item) next;
	void *item_template;
	void *table;
	void *flow;
	void *action_template[MAX_HWS_ACTION_TEMPLATE];
	uint8_t action_template_index;
	uint8_t pattern_template_index;
};

struct dpdk_hws_ctx {
	int nb_entries;
	LIST_HEAD(, dpdk_hws_item) hws_list;
};

struct dpdk_hws_cfg {
	uint32_t total_sessions;
	/**< total flows count */
	uint16_t queues;
	/**< queue id for each offload thread */
	uint16_t ctrl_queue_id;
	/**< queue id for ctrl queue */
	struct doca_flow_resources resource;
	/**< resource quota */
	uint32_t queue_depth;
	/**< Number of pre-configured queue_size */
	uint32_t nr_shared_resources[DOCA_FLOW_SHARED_RESOURCE_MAX];
	/**< total shared resource per type */
};

enum hws_flow_flag {
	HWS_FLOW_FLAG_PERSISTENT = (1 << 0),
	HWS_FLOW_FLAG_NON_PERSISTENT = (1 << 1),
};

struct dpdk_pipe;
struct dpdk_pipe_q_ctx;

void
dpdk_flow_hws_destroy();

int
dpdk_flow_hws_init(const struct dpdk_hws_cfg *cfg);

bool
dpdk_flow_hws_is_ctrl_entry(struct doca_flow_pipe_entry *entry);

struct dpdk_hws_item *
dpdk_flow_hws_get_pipe_first_ctx(struct dpdk_pipe *pipe);

/*
 * Should be removed, currently not possible due
 * https://redmine.mellanox.com/issues/3168220
 */
int
dpdk_flow_hws_mgmt_q_flow_destroy_legacy(uint16_t port_id,
					 uint32_t queue_id,
					 struct rte_flow_op_attr *q_ops_attr,
					 struct doca_flow_pipe_entry *entry,
					 void *cb, void *usr_ctx,
					 uint32_t flag);

int
dpdk_flow_hws_dequeue(uint16_t port_id,
			uint32_t queue_id,
			uint64_t timeout_ms,
			uint32_t max_processed_entries,
			bool force_push,
			struct doca_flow_pipe_entry *entry);

struct dpdk_hws_item *
dpdk_flow_hws_ctrl_table_create(struct doca_flow_port *port,
				 struct dpdk_pipe *pipe,
				 struct dpdk_pipe_q_ctx *pipe_q,
				 struct rte_flow_template_table_attr *tatr,
				 const struct rte_flow_item items[],
				 struct rte_flow_action **actions,
				 uint8_t nb_actions);

int
dpdk_flow_hws_ctrl_table_flow_destroy(struct doca_flow_port *port,
				      struct dpdk_hws_item *hws_entry);

struct rte_flow *
dpdk_flow_hws_ctrl_entry_create(struct doca_flow_port *port,
				struct dpdk_pipe *pipe,
				const struct rte_flow_item items[],
				const struct rte_flow_action actions[],
				void *usr_ctx, void *cb,
				struct doca_flow_pipe_entry *entry,
				struct dpdk_hws_item *hws_entry);

int
dpdk_hws_default_create(struct doca_flow_port *port);

int
dpdk_flow_hws_port_configure(struct doca_flow_port *port);

int
dpdk_flow_hws_table_destroy(struct doca_flow_pipe *pipe);

int
dpdk_flow_hws_templates_table_create(struct dpdk_pipe *dpdk_pipe, struct dpdk_pipe_q_ctx *pipe,
				     uint8_t nb_actions, uint32_t nb_flows);

struct rte_flow *
dpdk_flow_hws_enqueue(uint16_t port_id,
			uint32_t queue_id,
			struct rte_flow_op_attr *qatr,
			struct rte_flow_template_table *table,
			const struct rte_flow_item items[],
			uint8_t item_template_index,
			const struct rte_flow_action actions[],
			uint8_t action_template_index,
			uint32_t flags, void *usr_ctx,
			void *cb, struct doca_flow_pipe_entry *entry);

int
dpdk_flow_hws_create_encap_table(struct dpdk_pipe *dpdk_pipe,
		struct rte_flow_action_raw_decap *decap_conf,
		struct rte_flow_action_raw_encap *encap_conf);

int
dpdk_flow_hws_destroy_encap_table(struct dpdk_pipe *pipe, struct doca_flow_port *port);

int
dpdk_hws_flow_entries_process(struct doca_flow_port *port,
		uint16_t pipe_queue, uint64_t timeout,
		uint32_t max_processed_entries);
#endif
