/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPDK_PIPE_COMMON_H_
#define DPDK_PIPE_COMMON_H_

#include <stdio.h>

#include "dpdk_pipe_legacy.h"
#include "dpdk_table.h"

#ifdef __cplusplus
extern "C" {
#endif

struct dpdk_table_params;
struct doca_flow_pipe_entry;
struct doca_flow_monitor;
struct dpdk_table;

struct doca_flow_pipe *
dpdk_pipe_common_alloc(uint16_t nr_queues);

int
dpdk_pipe_common_verify(struct dpdk_pipe_cfg *pipe_cfg);

int
dpdk_pipe_common_entry_verify(struct doca_flow_pipe *pipe_legacy, uint16_t queue_id,
			      uint16_t action_idx, struct dpdk_uds_cfg *uds_cfg);

void
dpdk_pipe_common_resources_free(struct doca_flow_pipe *pipe_legacy);

void
dpdk_pipe_common_legacy_free(struct doca_flow_pipe *pipe_legacy);

void
dpdk_pipe_common_table_destroy(struct doca_flow_pipe *pipe_legacy);

int
dpdk_pipe_common_get_table(struct doca_flow_pipe *pipe,
	  uint8_t act_arr_idx,
	  struct dpdk_table *res);

void
dpdk_pipe_common_fill_flow_params(struct dpdk_flow_params *flow_params, struct rte_flow_item *items,
				  struct rte_flow_action *actions, struct dpdk_table *table,
				  struct rte_flow_attr *attr, uint16_t action_idx);

void
dpdk_pipe_common_fill_indexed_flow_params(struct dpdk_flow_params *flow_params, uint32_t index,
					  struct rte_flow_action *actions, struct dpdk_table *table,
					  struct rte_flow_attr *attr, uint16_t action_idx);

void
dpdk_pipe_common_entry_attach(uint16_t pipe_queue, struct doca_flow_pipe *pipe,
			      struct doca_flow_pipe_entry *entry);

void
dpdk_pipe_common_entry_detach(uint16_t pipe_queue, struct doca_flow_pipe *pipe,
			      struct doca_flow_pipe_entry *entry);

int
dpdk_flow_age_list_add(uint16_t queue, struct dpdk_pipe_q_ctx *pipe_q,
		       struct doca_flow_pipe_entry *entry, const struct doca_flow_monitor *monitor);

int
dpdk_pipe_common_queue_modify(enum engine_pipe_type pipe_type,
		       uint16_t pipe_queue,
		       struct dpdk_pipe *dpdk_pipe,
		       struct dpdk_pipe_q_ctx *pipe_q,
		       struct doca_flow_pipe_entry *entry,
		       const struct doca_flow_match *match,
		       const struct doca_flow_actions *actions,
		       const struct doca_flow_monitor *monitor,
		       const struct doca_flow_fwd *fwd,
		       uint32_t flags,
		       void *usr_ctx);

int
dpdk_pipe_common_queue_submit(struct doca_flow_pipe *pipe_legacy,
			      struct doca_flow_pipe_entry *entry, uint16_t queue_id,
			      struct dpdk_flow_params *flow_params,
			      dpdk_flow_completion_cb entry_completion_cb,
			      void *usr_ctx, bool no_wait);

int
dpdk_pipe_common_queue_remove(struct doca_flow_pipe_entry *entry,
			      dpdk_flow_completion_cb cb, void *usr_ctx);

void
dpdk_pipe_entry_release(struct doca_flow_pipe_entry *entry);

int
dpdk_pipe_basic_build(struct doca_flow_pipe *pipe, struct dpdk_pipe_cfg *pipe_cfg);

void
dpdk_pipe_common_fill_table_params(struct dpdk_table_params *table_params,
				   uint16_t port_id,
				   struct rte_flow_item items[],
				   struct rte_flow_action **actions,
				   struct rte_flow_action **actions_masks,
				   uint8_t nb_actions,
				   struct rte_flow_attr *attr,
				   uint32_t nb_flows,
				   enum dpdk_table_insertion_type insertion_type,
				   enum dpdk_table_hash_type hash_type);

int
dpdk_pipe_per_q_actions_build(enum engine_pipe_type type, struct dpdk_pipe *dpdk_pipe,
			      struct dpdk_pipe_q_ctx *pipe, struct dpdk_pipe_cfg *pipe_cfg);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* DPDK_PIPE_COMMON_H_ */
