/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPDK_PIPE_ITEMS_H_
#define DPDK_PIPE_ITEMS_H_

#include <stdio.h>

#include <rte_flow.h>
#include <rte_vxlan.h>
#include <rte_ethdev.h>
#include <rte_mtr.h>
#include <rte_gre.h>

#include "doca_flow.h"
#include <dpdk_shared_crypto.h>

#ifdef __cplusplus
extern "C" {
#endif

#define MAX_META_REG_COUNT (DOCA_FLOW_META_MAX / 4)

struct dpdk_pipe;
struct dpdk_pipe_q_ctx;

enum dpdk_modify_flags {
	DPDK_MODIFY_SMAC = (1 << 0),
	DPDK_MODIFY_DMAC = (1 << 1),
	DPDK_MODIFY_VLAN_TCI = (1 << 2),
	DPDK_MODIFY_SIP = (1 << 3),
	DPDK_MODIFY_DIP = (1 << 4),
	DPDK_MODIFY_SPORT = (1 << 5),
	DPDK_MODIFY_DPORT = (1 << 6),
	DPDK_MODIFY_TCP_FLAGS = (1 << 7),
	DPDK_MODIFY_GRE_KEY = (1 << 8),
	DPDK_MODIFY_VXLAN_VNI = (1 << 9),
	DPDK_MODIFY_GTP_TEID = (1 << 10),
	DPDK_MODIFY_ETH_TYPE = (1 << 11),
	DPDK_MODIFY_L4_PROTO = (1 << 12),
	DPDK_MODIFY_NISP = (1 << 13),
	DPDK_MODIFY_AUDP = (1 << 14),
	DPDK_MODIFY_NISP_SYNDROME = (1 << 15),
	DPDK_MODIFY_IPSEC_SYNDROME = (1 << 16),
	DPDK_MODIFY_ESP_SPI = (1 << 17),
	DPDK_MODIFY_ESP_SN = (1 << 18),
};

/* Tag is actually used in DPDK to fill metadata registers. */
struct dpdk_item_tag_data {
	struct rte_flow_item_tag spec;
	struct rte_flow_item_tag last;
	struct rte_flow_item_tag mask;
};

struct dpdk_item_meta_data {
	struct rte_flow_item_meta spec;
	struct rte_flow_item_meta last;
	struct rte_flow_item_meta mask;
};

struct dpdk_item_eth_data {
	bool is_outer;
	struct rte_flow_item_eth spec;
	struct rte_flow_item_eth last;
	struct rte_flow_item_eth mask;
};

struct dpdk_item_vlan_data {
	bool is_outer;
	struct rte_flow_item_vlan spec;
	struct rte_flow_item_vlan last;
	struct rte_flow_item_vlan mask;
};

struct dpdk_item_ipv4_data {
	bool is_outer;
	struct rte_flow_item_ipv4 spec;
	struct rte_flow_item_ipv4 last;
	struct rte_flow_item_ipv4 mask;
};

struct dpdk_item_ipv6_data {
	bool is_outer;
	struct rte_flow_item_ipv6 spec;
	struct rte_flow_item_ipv6 last;
	struct rte_flow_item_ipv6 mask;
};

struct dpdk_item_vxlan_data {
	struct rte_flow_item_vxlan spec;
	struct rte_flow_item_vxlan last;
	struct rte_flow_item_vxlan mask;
};

struct dpdk_item_gre_data {
	struct rte_flow_item_gre spec;
	struct rte_flow_item_gre last;
	struct rte_flow_item_gre mask;
};

struct dpdk_item_gre_key_data {
	doca_be32_t spec;
	doca_be32_t last;
	doca_be32_t mask;
};

struct dpdk_item_udp_data {
	bool is_outer;
	struct rte_flow_item_udp spec;
	struct rte_flow_item_udp last;
	struct rte_flow_item_udp mask;
};

struct dpdk_item_tcp_data {
	bool is_outer;
	struct rte_flow_item_tcp spec;
	struct rte_flow_item_tcp last;
	struct rte_flow_item_tcp mask;
};

struct dpdk_item_gtp_data {
	struct rte_flow_item_gtp spec;
	struct rte_flow_item_gtp last;
	struct rte_flow_item_gtp mask;
};

struct dpdk_item_ethdev_data {
	struct rte_flow_item_ethdev spec;
	struct rte_flow_item_ethdev mask;
};

struct dpdk_item_nisp_data {
	struct mlx5_flow_item_nisp spec;
	struct mlx5_flow_item_nisp last;
	struct mlx5_flow_item_nisp mask;
};

struct dpdk_item_audp_data {
	struct mlx5_flow_item_audp spec;
	struct mlx5_flow_item_audp last;
	struct mlx5_flow_item_audp mask;
};

struct dpdk_item_nisp_syndrome_data {
	struct mlx5_flow_item_nisp_syndrome spec;
	struct mlx5_flow_item_nisp_syndrome last;
	struct mlx5_flow_item_nisp_syndrome mask;
};

struct dpdk_item_ipsec_syndrome_data {
	struct mlx5_flow_item_ipsec_syndrome spec;
	struct mlx5_flow_item_ipsec_syndrome last;
	struct mlx5_flow_item_ipsec_syndrome mask;
};

struct dpdk_item_esp_data {
	struct rte_flow_item_esp spec;
	struct rte_flow_item_esp mask;
};

struct dpdk_item_meter_color {
	struct rte_flow_item_meter_color spec;
	struct rte_flow_item_meter_color mask;
};

struct dpdk_item_data {
	union {
		struct dpdk_item_meta_data meta;
		struct dpdk_item_tag_data tag;
		struct dpdk_item_eth_data eth;
		struct dpdk_item_vlan_data vlan;
		struct dpdk_item_ipv4_data ipv4;
		struct dpdk_item_ipv6_data ipv6;
		struct dpdk_item_vxlan_data vxlan;
		struct dpdk_item_tcp_data tcp;
		struct dpdk_item_udp_data udp;
		struct dpdk_item_gre_data gre;
		struct dpdk_item_gre_key_data gre_key;
		struct dpdk_item_gtp_data gtp;
		struct dpdk_item_ethdev_data port;
		struct dpdk_item_esp_data esp;
		struct dpdk_item_nisp_data nisp;
		struct dpdk_item_audp_data audp;
		struct dpdk_item_nisp_syndrome_data nisp_synd;
		struct dpdk_item_ipsec_syndrome_data ipsec_synd;
		struct dpdk_item_meter_color color;
	};
};

struct dpdk_item_entry {
	uint64_t flags;
	struct rte_flow_item *item;
	struct dpdk_item_data item_data;
	int (*modify_item)(struct dpdk_item_entry *entry,
		const struct doca_flow_match *match);
};

int
dpdk_pipe_items_meta_port_parse(const struct doca_flow_match *match,
				const struct doca_flow_match *mask,
				uint32_t *port_value,
				uint32_t *port_mask);

int
dpdk_pipe_matches_build(struct dpdk_pipe *dpdk_pipe,
		      struct dpdk_pipe_q_ctx *pipe_q,
		      const struct doca_flow_match *match,
		      const struct doca_flow_match *match_mask);

int
dpdk_pipe_matches_modify(struct dpdk_pipe *dpdk_pipe,
		       struct dpdk_pipe_q_ctx *pipe_q,
		       const struct doca_flow_match *match);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* DPDK_PIPE_ITEMS_H_ */
