/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DPDK_TABLE_MANAGER_H_
#define DPDK_TABLE_MANAGER_H_

#include <stdint.h>
#include <stdbool.h>

#ifdef __cplusplus
extern "C" {
#endif

struct dpdk_table_manager;
struct dpdk_table_params;
struct dpdk_pipe_q_ctx;
struct dpdk_table;

/**
 * @brief Dpdk table manager configuration
 */
struct dpdk_table_manager_cfg {
	uint32_t nr_tables;
	uint32_t unique_id;
};

/**
 * @brief Init dpdk table manager.
 */
void
dpdk_table_manager_init();

/**
 * @brief Creates a dpdk table manager.
 *
 * @param cfg
 * Pointer to configuration.
 * @return
 * Valid table manager pointer on success, else NULL.
 */
struct dpdk_table_manager *
dpdk_table_manager_create(struct dpdk_table_manager_cfg *cfg);

/**
 * @brief Destroys a dpdk table manager.
 *
 * @param table_manager
 * Pointer to a valid table manager.
 */
void
dpdk_table_manager_destroy(struct dpdk_table_manager *table_manager);

/**
 * @brief Allocate a table id.
 *
 * @param table_manager
 * Pointer to a valid table manager.
 * @param table_id
 * Pointer to a table id as output.
 * @return
 * 0 on success, otherwise error code.
 */
int
dpdk_table_manager_alloc_id(struct dpdk_table_manager *table_manager, uint32_t *table_id);

/**
 * @brief Gets the table by a table_id.
 *
 * @param table_manager
 * Pointer to a valid table manager.
 * @param table_id
 * Valid table id.
 * @return
 * On success, a pointer to a dpdk_table, otherwise - NULL.
 */
struct dpdk_table *
dpdk_table_manager_get_by_id(struct dpdk_table_manager *table_manager, uint32_t table_id);

/**
 * @brief Free a table id.
 *
 * @param table_manager
 * Pointer to a valid table manager.
 * @param table_id
 * Valid table id.
 * @return
 * 0 on success, otherwise error code.
 */
int
dpdk_table_manager_free_id(struct dpdk_table_manager *table_manager, uint32_t table_id);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* DPDK_TABLE_MANAGER_H_ */
