/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include <errno.h>

#include "engine_supported_types.h"
#include "engine_field_opcode.h"

int
engine_field_opcode_build_pkt(struct engine_field_opcode *opcode, uint8_t type, uint16_t focus,
			      uint16_t proto, uint8_t field)
{
	if (type >= DF_SUP_USER_DEFINED_TYPE_MAX)
		return -ENOTSUP;

	if (focus >= DF_SUP_FOCUS_MAX)
		return -ENOTSUP;

	if (proto >= DF_SUP_PROTO_MAX)
		return -ENOTSUP;

	opcode->opcode = 0;
	opcode->pkt.group = ENGINE_FIELD_OPCODE_GROUP_PACKET;
	opcode->pkt.type = type;
	opcode->pkt.focus = focus;
	opcode->pkt.proto = proto;
	opcode->pkt.field = field;
	return 0;
}

int
engine_field_opcode_build_flow(struct engine_field_opcode *opcode, uint32_t code)
{
	opcode->opcode = 0;
	opcode->flow.group = ENGINE_FIELD_OPCODE_GROUP_FLOW;
	opcode->flow.code = code;
	return 0;
}
