/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#include <errno.h>
#include <stddef.h>

#include "private/common_defines.h"
#include "engine_field_mapping.h"
#include "engine_field_opcode.h"
#include "engine_supported_types.h"
#include "engine_to_string.h"
#include "engine_layer.h"

#include <doca_log.h>

DOCA_LOG_REGISTER(engine_layer);


int
engine_layer_init(struct engine_layer_cfg *cfg)
{
	int rc;

	if (unlikely(!cfg))
		return -EINVAL;

	rc = engine_model_init(&cfg->model_cfg);
	if (unlikely(rc)) {
		DOCA_LOG_ERR("failed initializing engine layer - model rc=%d", rc);
		return rc;
	}

	rc = engine_to_string_init();
	if (unlikely(rc)) {
		DOCA_LOG_ERR("failed initializing engine layer - to string rc=%d", rc);
		return rc;
	}

	rc = engine_field_mapping_init();
	if (unlikely(rc)) {
		DOCA_LOG_ERR("failed initializing engine layer - field mapping rc=%d", rc);
		return rc;
	}

	rc = engine_shared_resources_init(&cfg->shared_res_cfg);
	if (unlikely(rc)) {
		DOCA_LOG_ERR("failed initializing engine layer - shared resource rc=%d", rc);
		return rc;
	}

	return 0;
}

void
engine_layer_destroy()
{
	engine_shared_resources_destroy();
	engine_field_mapping_destroy();
	engine_to_string_destroy();
	engine_model_destroy();
}
