/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef ENGINE_LAYER_H_
#define ENGINE_LAYER_H_

#include <stdint.h>

#include "engine_model.h"
#include "engine_to_string.h"
#include "engine_spinlock.h"

#include "engine_supported_types.h"
#include "engine_fcp.h"
#include "engine_uds.h"
#include "engine_field.h"
#include "engine_field_opcode.h"
#include "engine_field_mapping.h"

#include "engine_pipe.h"
#include "engine_bindable.h"
#include "engine_shared_resources.h"

#ifdef __cplusplus
extern "C" {
#endif

struct engine_layer_cfg {
	struct engine_model_cfg model_cfg;
	struct engine_shared_resources_cfg shared_res_cfg;
};

/**
 * @brief Initialize engine layer.
 *
 * @return
 * 0 on success, negative on failure.
 */
int
engine_layer_init(struct engine_layer_cfg *cfg);

/**
 * @brief Destroy engine layer.
 */
void
engine_layer_destroy();

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* ENGINE_LAYER_H_ */
