/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef ENGINE_PIPE_H_
#define ENGINE_PIPE_H_

#define ENGINE_PIPE_NAME_LEN 128

enum engine_pipe_type {
	ENGINE_PIPE_BASIC,
	/**< Flow pipe */
	ENGINE_PIPE_CONTROL,
	/**< Control pipe */
	ENGINE_PIPE_LPM,
	/**< lpm pipe */
	ENGINE_PIPE_CT,
	/**< Connection tracking pipe */
	ENGINE_PIPE_HASH,
	/**< Hash pipe */
	ENGINE_PIPE_ORDERED_LIST,
	/**< Ordered list pipe */
	ENGINE_PIPE_MAX,
	/**< max supported pipe type */
};

enum engine_pipe_fwd_type {
	ENGINE_FWD_NONE = 0,
	/**< No forward action be set */
	ENGINE_FWD_RSS,
	/**< Forwards packets to rss */
	ENGINE_FWD_PORT,
	/**< Forwards packets to one port */
	ENGINE_FWD_PIPE,
	/**< Forwards packets to another pipe */
	ENGINE_FWD_DROP,
	/**< Drops packets */
	ENGINE_FWD_CT,
	/**< Forwards packets to Connection Tracking */
};

struct engine_pipe_fwd {
	enum engine_pipe_fwd_type fwd_type;
	/**< forward action type */
	union {
		uint16_t port_id;
		/**< destination port id */
		uint32_t pipe_id;
		/**< destination pipe/group id */
	};
};

#endif /* ENGINE_PIPE_H_ */
