/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef ENGINE_PROTO_TYPES_H_
#define ENGINE_PROTO_TYPES_H_

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

#define DF_SUP_PROTO_FIELD_MAX (255)

enum engine_supported_user_defined_type {
	DF_SUP_USER_DEFINED_TYPE_MATCH,
	DF_SUP_USER_DEFINED_TYPE_MAX,
};

enum engine_supported_focus {
	DF_SUP_FOCUS_OUTER,
	DF_SUP_FOCUS_TUNNEL,
	DF_SUP_FOCUS_INNER,
	DF_SUP_FOCUS_MAX,
};

enum engine_supported_proto {
	DF_SUP_PROTO_ETH,
	DF_SUP_PROTO_ETH_VLAN,
	DF_SUP_PROTO_IPV4,
	DF_SUP_PROTO_IPV6,
	DF_SUP_PROTO_UDP,
	DF_SUP_PROTO_TCP,
	DF_SUP_PROTO_GRE,
	DF_SUP_PROTO_VXLAN,
	DF_SUP_PROTO_GTP,
	DF_SUP_PROTO_AUDP,
	DF_SUP_PROTO_NISP,
	DF_SUP_PROTO_ESP,
	DF_SUP_PROTO_MAX,
};

enum engine_supported_eth_fields {
	DF_SUP_PROTO_FIELD_ETH_DST_MAC,
	DF_SUP_PROTO_FIELD_ETH_SRC_MAC,
	DF_SUP_PROTO_FIELD_ETH_TYPE,
	DF_SUP_PROTO_FIELD_ETH_MAX,
};

enum engine_supported_eth_vlan_fields {
	DF_SUP_PROTO_FIELD_ETH_VLAN_TCI,
	DF_SUP_PROTO_FIELD_ETH_VLAN_MAX,
};

enum engine_supported_ipv4_fields {
	DF_SUP_PROTO_FIELD_IPV4_SRC_IP,
	DF_SUP_PROTO_FIELD_IPV4_DST_IP,
	DF_SUP_PROTO_FIELD_IPV4_NEXT_PROTO,
	DF_SUP_PROTO_FIELD_IPV4_MAX,
};

enum engine_supported_ipv6_fields {
	DF_SUP_PROTO_FIELD_IPV6_SRC_IP,
	DF_SUP_PROTO_FIELD_IPV6_DST_IP,
	DF_SUP_PROTO_FIELD_IPV6_NEXT_PROTO,
	DF_SUP_PROTO_FIELD_IPV6_MAX,
};

enum engine_supported_udp_fields {
	DF_SUP_PROTO_FIELD_UDP_SRC_PORT,
	DF_SUP_PROTO_FIELD_UDP_DST_PORT,
	DF_SUP_PROTO_FIELD_UDP_MAX,
};

enum engine_supported_tcp_fields {
	DF_SUP_PROTO_FIELD_TCP_SRC_PORT,
	DF_SUP_PROTO_FIELD_TCP_DST_PORT,
	DF_SUP_PROTO_FIELD_TCP_FLAGS,
	DF_SUP_PROTO_FIELD_TCP_MAX,
};

enum engine_supported_vxlan_fields {
	DF_SUP_PROTO_FIELD_VXLAN_TUN_ID,
	DF_SUP_PROTO_FIELD_VXLAN_MAX,
};

enum engine_supported_gre_fields {
	DF_SUP_PROTO_FIELD_GRE_KEY,
	DF_SUP_PROTO_FIELD_GRE_PROTOCOL,
	DF_SUP_PROTO_FIELD_GRE_MAX,
};

enum engine_supported_gtp_fields {
	DF_SUP_PROTO_FIELD_GTP_TEID,
	DF_SUP_PROTO_FIELD_GTP_MAX,
};

enum engine_supported_nisp_fields {
	DF_SUP_PROTO_FIELD_NISP_HEADER,
	DF_SUP_PROTO_FIELD_NISP_MAX,
};

enum engine_supported_audp_fields {
	DF_SUP_PROTO_FIELD_AUDP_HEADER,
	DF_SUP_PROTO_FIELD_AUDP_MAX,
};

enum engine_supported_esp_fields {
	DF_SUP_PROTO_FIELD_ESP_SPI,
	DF_SUP_PROTO_FIELD_ESP_ESN,
	DF_SUP_PROTO_FIELD_ESP_MAX,
};

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* ENGINE_PROTO_TYPES_H_ */
