/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef ENGINE_TO_STRING_H_
#define ENGINE_TO_STRING_H_

#include <stdbool.h>
#include <stdint.h>

#include "engine_field_opcode.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief Initialize engine to string
 *
 * @return
 * 0 on success, negative on failure.
 */
int
engine_to_string_init();

/**
 * @brief Initialize engine to string
 *
 * @return
 * 0 on success, negative on failure.
 */
void
engine_to_string_destroy();


/**
 * @brief opcode to string
 *
 * @param opcode
 * engine field opcode.
 * @param buff
 * Pointer to char buffer to fill.
 * @param len
 * Allowed buffer filling length from pointer.
 * @return
 * number of written bytes.
 */
int
engine_to_string_opcode(struct engine_field_opcode *opcode, char *buff, int len);

/**
 * @brief string to opcode
 *
 * @param buff
 * Pointer to a char buffer to convert.
 * @param len
 * Buffer length.
 * @param opcode
 * Pointer to a field opcode.
 * @return
 * 0 on success, negative on failure.
 */
int
engine_string_to_opcode(const char *buff, int len, struct engine_field_opcode *opcode);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* ENGINE_TO_STRING_H_ */
