/*
 * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef FLOW_AGE_H_
#define FLOW_AGE_H_

#include <stdio.h>

#include "doca_flow.h"

struct flow_age_cfg {
	void (*update_age_item)(void *age_ctx, struct flow_age_item *item);
	int (*query_pkts)(void *age_ctx, uint64_t *pkts);
};

struct flow_age_item {
	uint32_t pkts;
	uint32_t expr_time_in_sec;
	uint16_t aging_time_in_sec;
	void *age_ctx;
	uint64_t user_data;
};

struct flow_age_queue {
	uint32_t array_len;
	int32_t last_idx;/*last item idx, start from -1*/
	int32_t curr_idx;/*current check idx, start from 0*/
	struct flow_age_item item_array[0];
};

int
flow_age_init(const struct flow_age_cfg *cfg);

void
flow_age_pipe_destroy(struct doca_flow_pipe *pipe);

int
flow_age_pipe_init(struct doca_flow_pipe *pipe, uint32_t nb_flows);

int
flow_age_item_release(struct doca_flow_pipe_entry *entry);

struct flow_age_item *
flow_age_item_alloc(uint16_t queue, struct doca_flow_pipe_entry *entry,
		uint16_t time_out, uint64_t user_data);

int
flow_age_check(struct doca_flow_port *port, uint16_t queue,
		uint64_t quota, struct doca_flow_aged_query *entries, int len);

#endif /* FLOW_AGE_H_ */
