/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <errno.h>

#include <doca_log.h>

#include "lpm_utils.h"

DOCA_LOG_REGISTER(lpm_utils);

int
utl_get_mask_sequential_bits(const uint8_t *mask, uint8_t len, uint8_t *nr_seq_bits)
{
	bool zero_bit_found = false;
	int count = 0;
	uint32_t _8bits;

	if (mask == NULL || nr_seq_bits == NULL)
		return -EINVAL;

	if (*mask == 0)
		return -1;

	for (int i = 0; i < len; i++) {
		_8bits = *mask;

		if (_8bits == 0xFF) {
			if (zero_bit_found)
				return -1;
			count += 8;
		} else if (_8bits == 0) {
			zero_bit_found = true;
		} else {
			if (zero_bit_found)
				return -1;

			for (int bit = 7; bit >= 0 ; bit--) {
				if (utl_byte_is_bit_set(_8bits, bit) == 0)
					zero_bit_found = true;
				else {
					if (zero_bit_found == true)
						return -1;
					count++;
				}
			}
		}
		mask++;
	}
	*nr_seq_bits = count;
	return 0;
}

uint8_t
utl_subnet_mask_build_byte(uint8_t left_bits)
{
	return (~(UINT8_MAX >> left_bits));
}

/*
 * Takes an array of uint8_t (IPv6/IPv4 or MAC address) and masks it using the depth.
 * It leaves untouched one bit per unit in the depth variable
 * and set the rest to 0.
 * @param ip is IN OUT
 */
void
utl_field_mask_invoke(uint8_t *field_addr, uint8_t field_len, uint8_t mask_seq_bits)
{
	#define BITS_IN_BYTE   8

	int16_t  mask;
	int8_t left_bits = mask_seq_bits;

	for (int i = 0; i < field_len; i++) {
		if (left_bits <= 0) {
			field_addr[i] = 0;
			continue;
		}
		if (left_bits < BITS_IN_BYTE) {
			mask = (uint16_t)utl_subnet_mask_build_byte(left_bits);
			field_addr[i] = (uint8_t)(field_addr[i] & mask);
		}
		left_bits -= BITS_IN_BYTE;
	}
}
