/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DOCA_UTILS_H_
#define DOCA_UTILS_H_

/* calculates number of sequential set bits in the mask */
int
utl_get_mask_sequential_bits(const uint8_t *mask, uint8_t len, uint8_t *nr_seq_bits);

void
utl_field_mask_invoke(uint8_t *field_addr, uint8_t field_len, uint8_t mask_seq_bits);

uint8_t
utl_subnet_mask_build_byte(uint8_t left_bits);

__always_inline  uint8_t
utl_byte_is_bit_set(uint8_t byte, uint8_t bit)
{
	uint8_t bit_mask = 1 << bit;

	return (byte & bit_mask);
}
__always_inline  uint8_t
utl_get_nb_bytes_from_bits(uint8_t cidr_bits)
{
	return ((cidr_bits >> 3) + !!(cidr_bits & 0x07));
}

// return 0 if the first len bytes of array a and array b are equal
// return > 0 if the first non-matching character of a array is greater that of b array
// return < 0 if the first non-matching character of a array is lower that of b array

__always_inline int
utl_compare_array(const uint8_t *a, const uint8_t *b, uint8_t len)
{
	return memcmp(a, b, len);
}

#endif /* DOCA_UTILS_H_ */
