/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef ORDERED_LIST_H_
#define ORDERED_LIST_H_

#include "dpdk_pipe_legacy.h"

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Maximum number of elements (actions/action_descs/monitor) the user can specify per sequence type. */
#define ORDERED_LIST_MAX_ELEMENTS_IN_SEQUENCE 8
/* Maximum number of internal pipes per sequence type, worst case scenario is an item for each element. */
#define ORDERED_LIST_MAX_ITEMS_IN_SEQUENCE ORDERED_LIST_MAX_ELEMENTS_IN_SEQUENCE
/* Maximum number of sequence types. */
#define ORDERED_LIST_MAX_NUMBER_OF_SEQUENCES 4
/* Maximum number of total internal pipes. */
#define ORDERED_LIST_MAX_TOTAL_ITEMS \
	(ORDERED_LIST_MAX_ITEMS_IN_SEQUENCE * ORDERED_LIST_MAX_NUMBER_OF_SEQUENCES)

struct ordered_list_item {
	/* Index in the hash pipes array of the ordered list pipe. */
	uint32_t hash_pipe_idx;
	/* Index of the actions item in the ordered list (negative means "none"). */
	int actions_elt_idx;
	/* Index of the monitor item in the ordered list (negative means "none"). */
	int monitor_elt_idx;
};

struct ordered_list_cfg {
	struct ordered_list_item items[ORDERED_LIST_MAX_ITEMS_IN_SEQUENCE];
	uint32_t nb_items;
};

struct ordered_list_pipe_cfg {
	struct doca_flow_actions *actions_arr[ORDERED_LIST_MAX_TOTAL_ITEMS];
	struct doca_flow_action_descs *action_descs_arr[ORDERED_LIST_MAX_TOTAL_ITEMS];
	struct dpdk_pipe_cfg hash_pipe_cfg_arr[ORDERED_LIST_MAX_TOTAL_ITEMS];
	uint32_t nb_hash_pipes;
	struct {
		struct doca_flow_match match;
		struct doca_flow_match match_mask;
		struct doca_flow_fwd fwd;
		struct dpdk_pipe_cfg pipe_cfg;
	} frontend;
};

int
ordered_list_pipe_cfg_build(struct ordered_list_pipe_cfg *cfg,
		const struct doca_flow_ordered_list **ordered_lists,
		uint8_t nb_ordered_lists,
		struct ordered_list_cfg *lists);

#ifdef __cplusplus
} // extern "C"
#endif

#endif /* ORDERED_LIST_H_ */
