/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef PIPE_LPM_H_
#define PIPE_LPM_H_

#include "doca_flow.h"

enum lpm_oper_type {
  LPM_BUILD_ONLY,
  LPM_OFFLOAD_ONLY,
  LPM_PUSH
};

void *
lpm_create(const struct doca_flow_pipe_cfg *cfg,
		const struct doca_flow_fwd *fwd,
		const struct doca_flow_fwd *fwd_miss);

void
lpm_destroy(void *lpm);

int
lpm_entry_add(struct doca_flow_pipe_entry *doca_entry,
		uint16_t pipe_queue,
		const struct doca_flow_pipe *pipe,
		const struct doca_flow_match *match,
		const struct doca_flow_match *match_mask,
		const struct doca_flow_actions *actions,
		const struct doca_flow_fwd *fwd);

int
lpm_entry_lookup(const struct doca_flow_pipe *pipe,
		const struct doca_flow_match *match,
		const struct doca_flow_match *match_mask);

int
lpm_flush(uint16_t pipe_queue,
		struct doca_flow_pipe *pipe,
		enum lpm_oper_type flush_type);

int
lpm_entry_remove(uint16_t pipe_queue, void *usr_ctx,
		struct doca_flow_pipe_entry *entry);

struct doca_flow_pipe *
lpm_get_main_pipe(void *lpm);

void
lpm_extract_match(void *lpm, struct doca_flow_match *match);


uint32_t
lpm_get_number_of_built_rule(struct doca_flow_pipe *pipe);

int
lpm_lookup(struct doca_flow_pipe *pipe,	uint8_t *address);

#endif /* PIPE_LPM_H_ */
