/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef UTILS_BITMAP_H_
#define UTILS_BITMAP_H_

#include <stdint.h>
#include <stdbool.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief Bitmap to manage either indexes in an array or in some range.
 */
struct utils_bitmap;

/**
 * @brief Create a new bitmap object.
 *
 * @param nr_bits
 * Number of requested bits.
 * @return
 * Pointer to bitmap object initialized to zero.
 */
struct utils_bitmap *
utils_bitmap_create(uint32_t nr_bits);

/**
 * @brief Destroy the given bitmap object.
 *
 * @param bitmap
 * Pointer to bitmap for destroying.
 */
void
utils_bitmap_destroy(struct utils_bitmap *bitmap);

/**
 * @brief Get the bitmap value for specific bit.
 *
 * @param bitmap
 * Pointer to bitmap object.
 * @param bit
 * The specific bit to get.
 * @return
 * 1 if this bit is set.
 * 0 if this bit isn't set.
 * A negative errno for failure.
 */
int
utils_bitmap_get(struct utils_bitmap *bitmap, uint32_t bit);

/**
 * @brief Set specific bit.
 *
 * @param bitmap
 * Pointer to bitmap object.
 * @param bit
 * The specific bit to set.
 * @param on
 * Turn the bit on. When it false, the bit is turned off.
 * @return
 * 0 on success, a negative errno otherwise.
 */
int
utils_bitmap_set(struct utils_bitmap *bitmap, uint32_t bit, bool on);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* UTILS_BITMAP_H_ */
