/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */


#include <string.h>

#include <private/common_defines.h>
#include <doca_log.h>

#include "utils_field.h"

#define UTILS_FIELD_IGNORED_BYTE_VAL	(0x00)
#define UTILS_FIELD_CHANGEABLE_BYTE_VAL	(0xff)

DOCA_LOG_REGISTER(utils_field);

enum utils_field_type
utils_field_get_type(const void *field_ptr, uint16_t field_len)
{
	const uint8_t *data_array;
	uint16_t ignored_bytes;
	uint16_t changeable_bytes;
	uint16_t i;

	if (unlikely(!field_ptr)) {
		DOCA_LOG_ERR("failed getting field type - field pointer is null");
		return UTILS_FIELD_TYPE_MAX;
	}

	data_array = (uint8_t *)field_ptr;
	ignored_bytes = 0;
	changeable_bytes = 0;

	for (i = 0; i < field_len; i++) {

		if (data_array[i] == UTILS_FIELD_IGNORED_BYTE_VAL)
			ignored_bytes++;
		else if (data_array[i] == UTILS_FIELD_CHANGEABLE_BYTE_VAL)
			changeable_bytes++;
		else
			return UTILS_FIELD_TYPE_SPECIFIC;

		if (unlikely(ignored_bytes > 0 && changeable_bytes > 0))
			return UTILS_FIELD_TYPE_SPECIFIC;
	}
	if (changeable_bytes > 0)
		return UTILS_FIELD_TYPE_CHANGEABLE;
	return UTILS_FIELD_TYPE_IGNORED;
}

void
utils_field_set(void *field_ptr, uint16_t field_len, uint8_t val)
{
	if (unlikely(!field_ptr)) {
		DOCA_LOG_ERR("failed setting field data - pointer is null");
		return;
	}

	memset(field_ptr, val, field_len);
}

void
utils_field_copy(void *dst_field, const void *src_field, uint16_t field_len)
{

	if (unlikely(!src_field)) {
		DOCA_LOG_ERR("failed copy source field data - pointer is null");
		return;
	}

	if (unlikely(!dst_field)) {
		DOCA_LOG_ERR("failed copy destination field data - pointer is null");
		return;
	}

	memcpy(dst_field, src_field, field_len);
}

