/*
 * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef UTILS_FIELD_H_
#define UTILS_FIELD_H_

#include <stdint.h>
#include <stdbool.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief Field type
 */
enum utils_field_type {
	UTILS_FIELD_TYPE_IGNORED,
	/**< ignored field type */
	UTILS_FIELD_TYPE_SPECIFIC,
	/**< specific field type */
	UTILS_FIELD_TYPE_CHANGEABLE,
	/**< changeable field type */
	UTILS_FIELD_TYPE_MAX,
	/**< Max supported field type */
};


/**
 * @brief Get the field type
 *
 * @param field_ptr
 * Pointer to a field.
 * @param field_len
 * Field length in bytes.
 * @return
 * Field type.
 */
enum utils_field_type
utils_field_get_type(const void *field_ptr, uint16_t field_len);

/**
 * @brief Set a field to a constant value.
 *
 * @param field_ptr
 * Pointer to a field.
 * @param field_len
 * Field length in bytes.
 * @param val
 * Value to change each field byte.
 */
void
utils_field_set(void *field_ptr, uint16_t field_len, uint8_t val);

/**
 * @brief Copy field data to another field.
 *
 * @param dst_field
 * Pointer to a destination field - copy to.
 * @param src_field
 * Pointer to a source field - copy from.
 * @param field_len
 * Field length in bytes.
 */
void
utils_field_copy(void *dst_field, const void *src_field, uint16_t field_len);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* UTILS_FIELD_H_ */
